@@ -68,7 +68,7 @@ fn build_gcs_object_store(url: &Url) -> Result<Arc<dyn object_store::ObjectStore
     let host = get_host_name(url)?;
     let mut builder = GoogleCloudStorageBuilder::new().with_bucket_name(host);
 
-    if let Some(path) = env::var("GCP_SERVICE_ACCOUNT_PATH").ok() {
+    if let Ok(path) = env::var("GCP_SERVICE_ACCOUNT_PATH") {
         builder = builder.with_service_account_path(path);
     }
     match builder.build() {
@@ -78,10 +78,12 @@ fn build_gcs_object_store(url: &Url) -> Result<Arc<dyn object_store::ObjectStore
 }
 
 fn get_host_name(url: &Url) -> Result<&str> {
-    url.host_str().ok_or(DataFusionError::Execution(format!(
-        "Not able to parse hostname from url, {}",
-        url.as_str()
-    )))
+    url.host_str().ok_or_else(|| {
+        DataFusionError::Execution(format!(
+            "Not able to parse hostname from url, {}",
+            url.as_str()
+        ))
+    })
 }
 
 #[cfg(test)]
