@@ -42,8 +42,8 @@ fn create_context() -> Result<SessionContext> {
     let batch = RecordBatch::try_new(
         schema,
         vec![
-            Arc::new(Float32Array::from_slice(&[2.1, 3.1, 4.1, 5.1])),
-            Arc::new(Float64Array::from_slice(&[1.0, 2.0, 3.0, 4.0])),
+            Arc::new(Float32Array::from_slice([2.1, 3.1, 4.1, 5.1])),
+            Arc::new(Float64Array::from_slice([1.0, 2.0, 3.0, 4.0])),
         ],
     )?;
 
