@@ -23,6 +23,7 @@ use glob::Pattern;
 use itertools::Itertools;
 use object_store::path::Path;
 use object_store::{ObjectMeta, ObjectStore};
+use percent_encoding;
 use url::Url;
 
 /// A parsed URL identifying files for a listing table, see [`ListingTableUrl::parse`]
@@ -108,7 +109,9 @@ impl ListingTableUrl {
 
     /// Creates a new [`ListingTableUrl`] from a url and optional glob expression
     fn new(url: Url, glob: Option<Pattern>) -> Self {
-        let prefix = Path::parse(url.path()).expect("should be URL safe");
+        let decoded_path =
+            percent_encoding::percent_decode_str(url.path()).decode_utf8_lossy();
+        let prefix = Path::from(decoded_path.as_ref());
         Self { url, prefix, glob }
     }
 
@@ -246,6 +249,15 @@ mod tests {
         let url = ListingTableUrl::parse("file:///foo").unwrap();
         let child = Path::parse("/foob/bar").unwrap();
         assert!(url.strip_prefix(&child).is_none());
+
+        let url = ListingTableUrl::parse("file:///foo/ bar").unwrap();
+        assert_eq!(url.prefix.as_ref(), "foo/ bar");
+
+        let url = ListingTableUrl::parse("file:///foo/bar?").unwrap();
+        assert_eq!(url.prefix.as_ref(), "foo/bar");
+
+        let url = ListingTableUrl::parse("file:///foo/😺").unwrap();
+        assert_eq!(url.prefix.as_ref(), "foo/%F0%9F%98%BA");
     }
 
     #[test]
