@@ -22,9 +22,8 @@ use crate::{
     error::Result,
     physical_optimizer::PhysicalOptimizerRule,
     physical_plan::{
-        coalesce_batches::CoalesceBatchesExec, filter::FilterExec,
-        hash_join::HashJoinExec, repartition::RepartitionExec,
-        with_new_children_if_necessary,
+        coalesce_batches::CoalesceBatchesExec, filter::FilterExec, joins::HashJoinExec,
+        repartition::RepartitionExec, with_new_children_if_necessary,
     },
 };
 use std::sync::Arc;
@@ -47,7 +46,7 @@ impl PhysicalOptimizerRule for CoalesceBatches {
     fn optimize(
         &self,
         plan: Arc<dyn crate::physical_plan::ExecutionPlan>,
-        config: &crate::execution::context::SessionConfig,
+        _config: &crate::execution::context::SessionConfig,
     ) -> Result<Arc<dyn crate::physical_plan::ExecutionPlan>> {
         if plan.children().is_empty() {
             // leaf node, children cannot be replaced
@@ -57,7 +56,7 @@ impl PhysicalOptimizerRule for CoalesceBatches {
             let children = plan
                 .children()
                 .iter()
-                .map(|child| self.optimize(child.clone(), config))
+                .map(|child| self.optimize(child.clone(), _config))
                 .collect::<Result<Vec<_>>>()?;
             let plan = with_new_children_if_necessary(plan, children)?;
             // The goal here is to detect operators that could produce small batches and only
