@@ -321,6 +321,7 @@ mod tests {
     use futures::StreamExt;
 
     use super::*;
+    use crate::config::ConfigOptions;
     use crate::datasource::object_store::ObjectStoreUrl;
     use crate::physical_plan::metrics::ExecutionPlanMetricsSet;
     use crate::prelude::SessionContext;
@@ -366,6 +367,7 @@ mod tests {
             projection: None,
             limit,
             table_partition_cols: vec![],
+            config_options: ConfigOptions::new().into_shareable(),
         };
 
         let file_stream = FileStream::new(
