@@ -729,7 +729,7 @@ mod tests {
         // have to send at least one batch through to provoke error
         let batch = RecordBatch::try_from_iter(vec![(
             "my_awesome_field",
-            Arc::new(StringArray::from_slice(&["foo", "bar"])) as ArrayRef,
+            Arc::new(StringArray::from_slice(["foo", "bar"])) as ArrayRef,
         )])
         .unwrap();
 
@@ -788,7 +788,7 @@ mod tests {
         let task_ctx = session_ctx.task_ctx();
         let batch = RecordBatch::try_from_iter(vec![(
             "my_awesome_field",
-            Arc::new(StringArray::from_slice(&["foo", "bar"])) as ArrayRef,
+            Arc::new(StringArray::from_slice(["foo", "bar"])) as ArrayRef,
         )])
         .unwrap();
 
@@ -823,13 +823,13 @@ mod tests {
         let task_ctx = session_ctx.task_ctx();
         let batch1 = RecordBatch::try_from_iter(vec![(
             "my_awesome_field",
-            Arc::new(StringArray::from_slice(&["foo", "bar"])) as ArrayRef,
+            Arc::new(StringArray::from_slice(["foo", "bar"])) as ArrayRef,
         )])
         .unwrap();
 
         let batch2 = RecordBatch::try_from_iter(vec![(
             "my_awesome_field",
-            Arc::new(StringArray::from_slice(&["frob", "baz"])) as ArrayRef,
+            Arc::new(StringArray::from_slice(["frob", "baz"])) as ArrayRef,
         )])
         .unwrap();
 
@@ -978,25 +978,25 @@ mod tests {
     fn make_barrier_exec() -> BarrierExec {
         let batch1 = RecordBatch::try_from_iter(vec![(
             "my_awesome_field",
-            Arc::new(StringArray::from_slice(&["foo", "bar"])) as ArrayRef,
+            Arc::new(StringArray::from_slice(["foo", "bar"])) as ArrayRef,
         )])
         .unwrap();
 
         let batch2 = RecordBatch::try_from_iter(vec![(
             "my_awesome_field",
-            Arc::new(StringArray::from_slice(&["frob", "baz"])) as ArrayRef,
+            Arc::new(StringArray::from_slice(["frob", "baz"])) as ArrayRef,
         )])
         .unwrap();
 
         let batch3 = RecordBatch::try_from_iter(vec![(
             "my_awesome_field",
-            Arc::new(StringArray::from_slice(&["goo", "gar"])) as ArrayRef,
+            Arc::new(StringArray::from_slice(["goo", "gar"])) as ArrayRef,
         )])
         .unwrap();
 
         let batch4 = RecordBatch::try_from_iter(vec![(
             "my_awesome_field",
-            Arc::new(StringArray::from_slice(&["grob", "gaz"])) as ArrayRef,
+            Arc::new(StringArray::from_slice(["grob", "gaz"])) as ArrayRef,
         )])
         .unwrap();
 
@@ -1036,7 +1036,7 @@ mod tests {
         let task_ctx = session_ctx.task_ctx();
         let batch = RecordBatch::try_from_iter(vec![(
             "a",
-            Arc::new(StringArray::from_slice(&["foo"])) as ArrayRef,
+            Arc::new(StringArray::from_slice(["foo"])) as ArrayRef,
         )])
         .unwrap();
         let partitioning = Partitioning::Hash(
