@@ -538,14 +538,14 @@ async fn csv_query_count_star() {
 }
 
 #[tokio::test]
-async fn csv_query_count_one() {
+async fn csv_query_count_literal() {
     let ctx = SessionContext::new();
     register_aggregate_csv_by_sql(&ctx).await;
-    let sql = "SELECT COUNT(1) FROM aggregate_test_100";
+    let sql = "SELECT COUNT(2) FROM aggregate_test_100";
     let actual = execute_to_batches(&ctx, sql).await;
     let expected = vec![
         "+-----------------+",
-        "| COUNT(UInt8(1)) |",
+        "| COUNT(Int64(2)) |",
         "+-----------------+",
         "| 100             |",
         "+-----------------+",
@@ -1475,7 +1475,7 @@ async fn aggregate_timestamps_min() -> Result<()> {
         "+----------------------------+----------------------------+-------------------------+---------------------+",
         "| MIN(t.nanos)               | MIN(t.micros)              | MIN(t.millis)           | MIN(t.secs)         |",
         "+----------------------------+----------------------------+-------------------------+---------------------+",
-        "| 2011-12-13 11:13:10.123450 | 2011-12-13 11:13:10.123450 | 2011-12-13 11:13:10.123 | 2011-12-13 11:13:10 |",
+        "| 2011-12-13T11:13:10.123450 | 2011-12-13T11:13:10.123450 | 2011-12-13T11:13:10.123 | 2011-12-13T11:13:10 |",
         "+----------------------------+----------------------------+-------------------------+---------------------+",
     ];
     assert_batches_sorted_eq!(expected, &results);
@@ -1498,7 +1498,7 @@ async fn aggregate_timestamps_max() -> Result<()> {
         "+-------------------------+-------------------------+-------------------------+---------------------+",
         "| MAX(t.nanos)            | MAX(t.micros)           | MAX(t.millis)           | MAX(t.secs)         |",
         "+-------------------------+-------------------------+-------------------------+---------------------+",
-        "| 2021-01-01 05:11:10.432 | 2021-01-01 05:11:10.432 | 2021-01-01 05:11:10.432 | 2021-01-01 05:11:10 |",
+        "| 2021-01-01T05:11:10.432 | 2021-01-01T05:11:10.432 | 2021-01-01T05:11:10.432 | 2021-01-01T05:11:10 |",
         "+-------------------------+-------------------------+-------------------------+---------------------+",
     ];
     assert_batches_sorted_eq!(expected, &results);
@@ -2016,11 +2016,11 @@ async fn simple_avg() -> Result<()> {
 
     let batch1 = RecordBatch::try_new(
         Arc::new(schema.clone()),
-        vec![Arc::new(Int32Array::from_slice(&[1, 2, 3]))],
+        vec![Arc::new(Int32Array::from_slice([1, 2, 3]))],
     )?;
     let batch2 = RecordBatch::try_new(
         Arc::new(schema.clone()),
-        vec![Arc::new(Int32Array::from_slice(&[4, 5]))],
+        vec![Arc::new(Int32Array::from_slice([4, 5]))],
     )?;
 
     let ctx = SessionContext::new();
@@ -2051,11 +2051,11 @@ async fn simple_mean() -> Result<()> {
 
     let batch1 = RecordBatch::try_new(
         Arc::new(schema.clone()),
-        vec![Arc::new(Int32Array::from_slice(&[1, 2, 3]))],
+        vec![Arc::new(Int32Array::from_slice([1, 2, 3]))],
     )?;
     let batch2 = RecordBatch::try_new(
         Arc::new(schema.clone()),
-        vec![Arc::new(Int32Array::from_slice(&[4, 5]))],
+        vec![Arc::new(Int32Array::from_slice([4, 5]))],
     )?;
 
     let ctx = SessionContext::new();
