@@ -527,8 +527,8 @@ async fn group_by_date_trunc() -> Result<()> {
         "+---------------------+--------------+",
         "| week                | SUM(test.c2) |",
         "+---------------------+--------------+",
-        "| 2020-12-07 00:00:00 | 24           |",
-        "| 2020-12-14 00:00:00 | 156          |",
+        "| 2020-12-07T00:00:00 | 24           |",
+        "| 2020-12-14T00:00:00 | 156          |",
         "+---------------------+--------------+",
     ];
     assert_batches_sorted_eq!(expected, &results);
@@ -654,13 +654,13 @@ async fn group_by_dictionary() {
         .expect("ran plan correctly");
 
         let expected = vec![
-            "+-----+------------------------+",
-            "| val | COUNT(DISTINCT t.dict) |",
-            "+-----+------------------------+",
-            "| 1   | 2                      |",
-            "| 2   | 2                      |",
-            "| 4   | 1                      |",
-            "+-----+------------------------+",
+            "+-------+------------------------+",
+            "| t.val | COUNT(DISTINCT t.dict) |",
+            "+-------+------------------------+",
+            "| 1     | 2                      |",
+            "| 2     | 2                      |",
+            "| 4     | 1                      |",
+            "+-------+------------------------+",
         ];
         assert_batches_sorted_eq!(expected, &results);
     }
