@@ -48,6 +48,28 @@ async fn parquet_query() {
     assert_batches_eq!(expected, &actual);
 }
 
+#[tokio::test]
+async fn fixed_size_binary_columns() {
+    let ctx = SessionContext::new();
+    ctx.register_parquet(
+        "t0",
+        "tests/parquet/test_binary.parquet",
+        ParquetReadOptions::default(),
+    )
+    .await
+    .unwrap();
+    let sql = "SELECT ids FROM t0 ORDER BY ids";
+    let plan = ctx.create_logical_plan(sql).unwrap();
+    let plan = ctx.optimize(&plan).unwrap();
+    let plan = ctx.create_physical_plan(&plan).await.unwrap();
+    let task_ctx = ctx.task_ctx();
+    let results = collect(plan, task_ctx).await.unwrap();
+    for batch in results {
+        assert_eq!(466, batch.num_rows());
+        assert_eq!(1, batch.num_columns());
+    }
+}
+
 #[tokio::test]
 async fn parquet_single_nan_schema() {
     let ctx = SessionContext::new();
@@ -190,16 +212,16 @@ async fn parquet_query_with_max_min() {
 
     if let Ok(()) = fs::create_dir(table_path) {
         let filename = "foo.parquet";
-        let path = table_path.join(&filename);
+        let path = table_path.join(filename);
         let file = fs::File::create(path).unwrap();
         let mut writer =
             ArrowWriter::try_new(file.try_clone().unwrap(), schema.clone(), None)
                 .unwrap();
 
         // create mock record batch
-        let c1s = Arc::new(Int32Array::from_slice(&[1, 2, 3]));
-        let c2s = Arc::new(StringArray::from_slice(&["aaa", "bbb", "ccc"]));
-        let c3s = Arc::new(Int64Array::from_slice(&[100, 200, 300]));
+        let c1s = Arc::new(Int32Array::from_slice([1, 2, 3]));
+        let c2s = Arc::new(StringArray::from_slice(["aaa", "bbb", "ccc"]));
+        let c3s = Arc::new(Int64Array::from_slice([100, 200, 300]));
         let c4s = Arc::new(Date32Array::from(vec![Some(1), Some(2), Some(3)]));
         let rec_batch =
             RecordBatch::try_new(schema.clone(), vec![c1s, c2s, c3s, c4s]).unwrap();
