@@ -289,10 +289,10 @@ impl LogicalPlanBuilder {
     /// Apply a filter
     pub fn filter(&self, expr: impl Into<Expr>) -> Result<Self> {
         let expr = normalize_col(expr.into(), &self.plan)?;
-        Ok(Self::from(LogicalPlan::Filter(Filter {
-            predicate: expr,
-            input: Arc::new(self.plan.clone()),
-        })))
+        Ok(Self::from(LogicalPlan::Filter(Filter::try_new(
+            expr,
+            Arc::new(self.plan.clone()),
+        )?)))
     }
 
     /// Limit the number of rows returned
@@ -323,7 +323,6 @@ impl LogicalPlanBuilder {
 
     /// Add missing sort columns to all downstream projection
     fn add_missing_columns(
-        &self,
         curr_plan: LogicalPlan,
         missing_cols: &[Column],
     ) -> Result<LogicalPlan> {
@@ -354,7 +353,7 @@ impl LogicalPlanBuilder {
                     .inputs()
                     .into_iter()
                     .map(|input_plan| {
-                        self.add_missing_columns((*input_plan).clone(), missing_cols)
+                        Self::add_missing_columns((*input_plan).clone(), missing_cols)
                     })
                     .collect::<Result<Vec<_>>>()?;
 
@@ -399,7 +398,7 @@ impl LogicalPlanBuilder {
             })));
         }
 
-        let plan = self.add_missing_columns(self.plan.clone(), &missing_cols)?;
+        let plan = Self::add_missing_columns(self.plan.clone(), &missing_cols)?;
         let sort_plan = LogicalPlan::Sort(Sort {
             expr: normalize_cols(exprs, &plan)?,
             input: Arc::new(plan.clone()),
@@ -743,7 +742,7 @@ impl LogicalPlanBuilder {
         LogicalPlanBuilder::intersect_or_except(
             left_plan,
             right_plan,
-            JoinType::Semi,
+            JoinType::LeftSemi,
             is_all,
         )
     }
@@ -757,7 +756,7 @@ impl LogicalPlanBuilder {
         LogicalPlanBuilder::intersect_or_except(
             left_plan,
             right_plan,
-            JoinType::Anti,
+            JoinType::LeftAnti,
             is_all,
         )
     }
@@ -823,10 +822,14 @@ pub fn build_join_schema(
             // left then right
             left_fields.chain(right_fields).cloned().collect()
         }
-        JoinType::Semi | JoinType::Anti => {
+        JoinType::LeftSemi | JoinType::LeftAnti => {
             // Only use the left side for the schema
             left.fields().clone()
         }
+        JoinType::RightSemi | JoinType::RightAnti => {
+            // Only use the right side for the schema
+            right.fields().clone()
+        }
     };
 
     let mut metadata = left.metadata().clone();
@@ -841,7 +844,7 @@ pub(crate) fn validate_unique_names<'a>(
 ) -> Result<()> {
     let mut unique_names = HashMap::new();
     expressions.into_iter().enumerate().try_for_each(|(position, expr)| {
-        let name = expr.name()?;
+        let name = expr.display_name()?;
         match unique_names.get(&name) {
             None => {
                 unique_names.insert(name, (position, expr));
