@@ -21,7 +21,7 @@
 use crate::{OptimizerConfig, OptimizerRule};
 use datafusion_common::{Result, ScalarValue};
 use datafusion_expr::{
-    logical_plan::{EmptyRelation, Filter, LogicalPlan},
+    logical_plan::{EmptyRelation, LogicalPlan},
     utils::from_plan,
     Expr,
 };
@@ -41,28 +41,30 @@ impl OptimizerRule for EliminateFilter {
     fn optimize(
         &self,
         plan: &LogicalPlan,
-        optimizer_config: &mut OptimizerConfig,
+        _optimizer_config: &mut OptimizerConfig,
     ) -> Result<LogicalPlan> {
-        match plan {
-            LogicalPlan::Filter(Filter {
-                predicate: Expr::Literal(ScalarValue::Boolean(Some(v))),
-                input,
-            }) => {
-                if !*v {
-                    Ok(LogicalPlan::EmptyRelation(EmptyRelation {
-                        produce_one_row: false,
-                        schema: input.schema().clone(),
-                    }))
-                } else {
-                    self.optimize(input, optimizer_config)
+        let predicate_and_input = match plan {
+            LogicalPlan::Filter(filter) => match filter.predicate() {
+                Expr::Literal(ScalarValue::Boolean(Some(v))) => {
+                    Some((*v, filter.input()))
                 }
-            }
-            _ => {
+                _ => None,
+            },
+            _ => None,
+        };
+
+        match predicate_and_input {
+            Some((true, input)) => self.optimize(input, _optimizer_config),
+            Some((false, input)) => Ok(LogicalPlan::EmptyRelation(EmptyRelation {
+                produce_one_row: false,
+                schema: input.schema().clone(),
+            })),
+            None => {
                 // Apply the optimization to all inputs of the plan
                 let inputs = plan.inputs();
                 let new_inputs = inputs
                     .iter()
-                    .map(|plan| self.optimize(plan, optimizer_config))
+                    .map(|plan| self.optimize(plan, _optimizer_config))
                     .collect::<Result<Vec<_>>>()?;
 
                 from_plan(plan, &plan.expressions(), &new_inputs)
