@@ -20,9 +20,9 @@ pub mod decorrelate_where_exists;
 pub mod decorrelate_where_in;
 pub mod eliminate_filter;
 pub mod eliminate_limit;
-pub mod expr_simplifier;
 pub mod filter_null_join_keys;
 pub mod filter_push_down;
+pub mod inline_table_scan;
 pub mod limit_push_down;
 pub mod optimizer;
 pub mod projection_push_down;
@@ -41,3 +41,4 @@ pub mod test;
 pub mod unwrap_cast_in_comparison;
 
 pub use optimizer::{OptimizerConfig, OptimizerRule};
+pub use utils::optimize_children;
