@@ -20,7 +20,9 @@
 use crate::{OptimizerConfig, OptimizerRule};
 use datafusion_common::{DataFusionError, Result};
 use datafusion_expr::{
-    logical_plan::{Join, JoinType, Limit, LogicalPlan, Projection, TableScan, Union},
+    logical_plan::{
+        Join, JoinType, Limit, LogicalPlan, Projection, Sort, TableScan, Union,
+    },
     utils::from_plan,
 };
 use std::sync::Arc;
@@ -41,10 +43,7 @@ impl LimitPushDown {
 /// when traversing down related to "limit push down".
 enum Ancestor {
     /// Limit
-    FromLimit {
-        skip: Option<usize>,
-        fetch: Option<usize>,
-    },
+    FromLimit { skip: usize, fetch: Option<usize> },
     /// Other nodes that don't affect the adjustment of "Limit"
     NotRelevant,
 }
@@ -90,8 +89,7 @@ fn limit_push_down(
                 } => {
                     if let Some(fetch) = current_fetch {
                         // extend ancestor's fetch
-                        let ancestor_fetch =
-                            ancestor_fetch.map(|f| f + ancestor_skip.unwrap_or(0));
+                        let ancestor_fetch = ancestor_fetch.map(|f| f + ancestor_skip);
 
                         let new_current_fetch =
                             ancestor_fetch.map_or(*fetch, |x| std::cmp::min(x, *fetch));
@@ -100,7 +98,7 @@ fn limit_push_down(
                     } else {
                         // we dont have a "fetch", and we can push down our parent's "fetch"
                         // extend ancestor's fetch
-                        ancestor_fetch.map(|f| f + ancestor_skip.unwrap_or(0))
+                        ancestor_fetch.map(|f| f + ancestor_skip)
                     }
                 }
                 _ => *current_fetch,
@@ -139,8 +137,7 @@ fn limit_push_down(
                 ..
             },
         ) => {
-            let ancestor_fetch =
-                ancestor_skip.map_or(ancestor_fetch, |x| x + ancestor_fetch);
+            let ancestor_fetch = ancestor_fetch + ancestor_skip;
             Ok(LogicalPlan::TableScan(TableScan {
                 table_name: table_name.clone(),
                 source: source.clone(),
@@ -187,18 +184,17 @@ fn limit_push_down(
             },
         ) => {
             // Push down limit through UNION
-            let ancestor_fetch =
-                ancestor_skip.map_or(ancestor_fetch, |x| x + ancestor_fetch);
+            let ancestor_fetch = ancestor_fetch + ancestor_skip;
             let new_inputs = inputs
                 .iter()
                 .map(|x| {
                     Ok(Arc::new(LogicalPlan::Limit(Limit {
-                        skip: None,
+                        skip: 0,
                         fetch: Some(ancestor_fetch),
                         input: Arc::new(limit_push_down(
                             _optimizer,
                             Ancestor::FromLimit {
-                                skip: None,
+                                skip: 0,
                                 fetch: Some(ancestor_fetch),
                             },
                             x,
@@ -221,8 +217,7 @@ fn limit_push_down(
                 ..
             },
         ) => {
-            let ancestor_fetch =
-                ancestor_skip.map_or(ancestor_fetch, |x| x + ancestor_fetch);
+            let ancestor_fetch = ancestor_fetch + ancestor_skip;
             match join_type {
                 JoinType::Left => {
                     //if LeftOuter join push limit to left
@@ -254,6 +249,25 @@ fn limit_push_down(
                 ),
             }
         }
+        (
+            LogicalPlan::Sort(Sort { expr, input, fetch }),
+            Ancestor::FromLimit {
+                skip: ancestor_skip,
+                fetch: Some(ancestor_fetch),
+                ..
+            },
+        ) => {
+            // Update Sort `fetch`, but simply recurse through children (sort should receive all input for sorting)
+            let input = push_down_children_limit(_optimizer, _optimizer_config, input)?;
+            let sort_fetch = ancestor_skip + ancestor_fetch;
+            let plan = LogicalPlan::Sort(Sort {
+                expr: expr.clone(),
+                input: Arc::new(input),
+                fetch: Some(fetch.map(|f| f.min(sort_fetch)).unwrap_or(sort_fetch)),
+            });
+            Ok(plan)
+        }
+
         // For other nodes we can't push down the limit
         // But try to recurse and find other limit nodes to push down
         _ => push_down_children_limit(_optimizer, _optimizer_config, plan),
@@ -282,7 +296,7 @@ fn generate_push_down_join(
             left: Arc::new(limit_push_down(
                 _optimizer,
                 Ancestor::FromLimit {
-                    skip: None,
+                    skip: 0,
                     fetch: left_limit,
                 },
                 left.as_ref(),
@@ -291,7 +305,7 @@ fn generate_push_down_join(
             right: Arc::new(limit_push_down(
                 _optimizer,
                 Ancestor::FromLimit {
-                    skip: None,
+                    skip: 0,
                     fetch: right_limit,
                 },
                 right.as_ref(),
@@ -347,6 +361,8 @@ impl OptimizerRule for LimitPushDown {
 
 #[cfg(test)]
 mod test {
+    use std::vec;
+
     use super::*;
     use crate::test::*;
     use datafusion_expr::{
@@ -370,13 +386,13 @@ mod test {
 
         let plan = LogicalPlanBuilder::from(table_scan)
             .project(vec![col("a")])?
-            .limit(None, Some(1000))?
+            .limit(0, Some(1000))?
             .build()?;
 
         // Should push the limit down to table provider
         // When it has a select
-        let expected = "Limit: skip=None, fetch=1000\
-        \n  Projection: #test.a\
+        let expected = "Limit: skip=0, fetch=1000\
+        \n  Projection: test.a\
         \n    TableScan: test, fetch=1000";
 
         assert_optimized_plan_eq(&plan, expected);
@@ -388,15 +404,15 @@ mod test {
         let table_scan = test_table_scan()?;
 
         let plan = LogicalPlanBuilder::from(table_scan)
-            .limit(None, Some(1000))?
-            .limit(None, Some(10))?
+            .limit(0, Some(1000))?
+            .limit(0, Some(10))?
             .build()?;
 
         // Should push down the smallest limit
         // Towards table scan
         // This rule doesn't replace multiple limits
-        let expected = "Limit: skip=None, fetch=10\
-        \n  Limit: skip=None, fetch=10\
+        let expected = "Limit: skip=0, fetch=10\
+        \n  Limit: skip=0, fetch=10\
         \n    TableScan: test, fetch=10";
 
         assert_optimized_plan_eq(&plan, expected);
@@ -410,12 +426,12 @@ mod test {
 
         let plan = LogicalPlanBuilder::from(table_scan)
             .aggregate(vec![col("a")], vec![max(col("b"))])?
-            .limit(None, Some(1000))?
+            .limit(0, Some(1000))?
             .build()?;
 
         // Limit should *not* push down aggregate node
-        let expected = "Limit: skip=None, fetch=1000\
-        \n  Aggregate: groupBy=[[#test.a]], aggr=[[MAX(#test.b)]]\
+        let expected = "Limit: skip=0, fetch=1000\
+        \n  Aggregate: groupBy=[[test.a]], aggr=[[MAX(test.b)]]\
         \n    TableScan: test";
 
         assert_optimized_plan_eq(&plan, expected);
@@ -429,15 +445,15 @@ mod test {
 
         let plan = LogicalPlanBuilder::from(table_scan.clone())
             .union(LogicalPlanBuilder::from(table_scan).build()?)?
-            .limit(None, Some(1000))?
+            .limit(0, Some(1000))?
             .build()?;
 
         // Limit should push down through union
-        let expected = "Limit: skip=None, fetch=1000\
+        let expected = "Limit: skip=0, fetch=1000\
         \n  Union\
-        \n    Limit: skip=None, fetch=1000\
+        \n    Limit: skip=0, fetch=1000\
         \n      TableScan: test, fetch=1000\
-        \n    Limit: skip=None, fetch=1000\
+        \n    Limit: skip=0, fetch=1000\
         \n      TableScan: test, fetch=1000";
 
         assert_optimized_plan_eq(&plan, expected);
@@ -445,20 +461,58 @@ mod test {
         Ok(())
     }
 
+    #[test]
+    fn limit_push_down_sort() -> Result<()> {
+        let table_scan = test_table_scan()?;
+
+        let plan = LogicalPlanBuilder::from(table_scan)
+            .sort(vec![col("a")])?
+            .limit(0, Some(10))?
+            .build()?;
+
+        // Should push down limit to sort
+        let expected = "Limit: skip=0, fetch=10\
+        \n  Sort: test.a, fetch=10\
+        \n    TableScan: test";
+
+        assert_optimized_plan_eq(&plan, expected);
+
+        Ok(())
+    }
+
+    #[test]
+    fn limit_push_down_sort_skip() -> Result<()> {
+        let table_scan = test_table_scan()?;
+
+        let plan = LogicalPlanBuilder::from(table_scan)
+            .sort(vec![col("a")])?
+            .limit(5, Some(10))?
+            .build()?;
+
+        // Should push down limit to sort
+        let expected = "Limit: skip=5, fetch=10\
+        \n  Sort: test.a, fetch=15\
+        \n    TableScan: test";
+
+        assert_optimized_plan_eq(&plan, expected);
+
+        Ok(())
+    }
+
     #[test]
     fn multi_stage_limit_recurses_to_deeper_limit() -> Result<()> {
         let table_scan = test_table_scan()?;
 
         let plan = LogicalPlanBuilder::from(table_scan)
-            .limit(None, Some(1000))?
+            .limit(0, Some(1000))?
             .aggregate(vec![col("a")], vec![max(col("b"))])?
-            .limit(None, Some(10))?
+            .limit(0, Some(10))?
             .build()?;
 
         // Limit should use deeper LIMIT 1000, but Limit 10 shouldn't push down aggregation
-        let expected = "Limit: skip=None, fetch=10\
-        \n  Aggregate: groupBy=[[#test.a]], aggr=[[MAX(#test.b)]]\
-        \n    Limit: skip=None, fetch=1000\
+        let expected = "Limit: skip=0, fetch=10\
+        \n  Aggregate: groupBy=[[test.a]], aggr=[[MAX(test.b)]]\
+        \n    Limit: skip=0, fetch=1000\
         \n      TableScan: test, fetch=1000";
 
         assert_optimized_plan_eq(&plan, expected);
@@ -470,7 +524,7 @@ mod test {
     fn limit_pushdown_should_not_pushdown_limit_with_offset_only() -> Result<()> {
         let table_scan = test_table_scan()?;
         let plan = LogicalPlanBuilder::from(table_scan)
-            .limit(Some(10), None)?
+            .limit(10, None)?
             .build()?;
 
         // Should not push any limit down to table provider
@@ -488,13 +542,13 @@ mod test {
 
         let plan = LogicalPlanBuilder::from(table_scan)
             .project(vec![col("a")])?
-            .limit(Some(10), Some(1000))?
+            .limit(10, Some(1000))?
             .build()?;
 
         // Should push the limit down to table provider
         // When it has a select
         let expected = "Limit: skip=10, fetch=1000\
-        \n  Projection: #test.a\
+        \n  Projection: test.a\
         \n    TableScan: test, fetch=1010";
 
         assert_optimized_plan_eq(&plan, expected);
@@ -508,13 +562,13 @@ mod test {
 
         let plan = LogicalPlanBuilder::from(table_scan)
             .project(vec![col("a")])?
-            .limit(None, Some(1000))?
-            .limit(Some(10), None)?
+            .limit(0, Some(1000))?
+            .limit(10, None)?
             .build()?;
 
         let expected = "Limit: skip=10, fetch=None\
-        \n  Limit: skip=None, fetch=1000\
-        \n    Projection: #test.a\
+        \n  Limit: skip=0, fetch=1000\
+        \n    Projection: test.a\
         \n      TableScan: test, fetch=1000";
 
         assert_optimized_plan_eq(&plan, expected);
@@ -528,13 +582,13 @@ mod test {
 
         let plan = LogicalPlanBuilder::from(table_scan)
             .project(vec![col("a")])?
-            .limit(Some(10), None)?
-            .limit(None, Some(1000))?
+            .limit(10, None)?
+            .limit(0, Some(1000))?
             .build()?;
 
-        let expected = "Limit: skip=None, fetch=1000\
+        let expected = "Limit: skip=0, fetch=1000\
         \n  Limit: skip=10, fetch=1000\
-        \n    Projection: #test.a\
+        \n    Projection: test.a\
         \n      TableScan: test, fetch=1010";
 
         assert_optimized_plan_eq(&plan, expected);
@@ -547,16 +601,16 @@ mod test {
         let table_scan = test_table_scan()?;
 
         let plan = LogicalPlanBuilder::from(table_scan)
-            .limit(Some(10), None)?
-            .limit(None, Some(1000))?
-            .limit(None, Some(10))?
+            .limit(10, None)?
+            .limit(0, Some(1000))?
+            .limit(0, Some(10))?
             .build()?;
 
         // Should push down the smallest limit
         // Towards table scan
         // This rule doesn't replace multiple limits
-        let expected = "Limit: skip=None, fetch=10\
-        \n  Limit: skip=None, fetch=10\
+        let expected = "Limit: skip=0, fetch=10\
+        \n  Limit: skip=0, fetch=10\
         \n    Limit: skip=10, fetch=10\
         \n      TableScan: test, fetch=20";
 
@@ -571,12 +625,12 @@ mod test {
 
         let plan = LogicalPlanBuilder::from(table_scan)
             .aggregate(vec![col("a")], vec![max(col("b"))])?
-            .limit(Some(10), Some(1000))?
+            .limit(10, Some(1000))?
             .build()?;
 
         // Limit should *not* push down aggregate node
         let expected = "Limit: skip=10, fetch=1000\
-        \n  Aggregate: groupBy=[[#test.a]], aggr=[[MAX(#test.b)]]\
+        \n  Aggregate: groupBy=[[test.a]], aggr=[[MAX(test.b)]]\
         \n    TableScan: test";
 
         assert_optimized_plan_eq(&plan, expected);
@@ -590,15 +644,15 @@ mod test {
 
         let plan = LogicalPlanBuilder::from(table_scan.clone())
             .union(LogicalPlanBuilder::from(table_scan).build()?)?
-            .limit(Some(10), Some(1000))?
+            .limit(10, Some(1000))?
             .build()?;
 
         // Limit should push down through union
         let expected = "Limit: skip=10, fetch=1000\
         \n  Union\
-        \n    Limit: skip=None, fetch=1010\
+        \n    Limit: skip=0, fetch=1010\
         \n      TableScan: test, fetch=1010\
-        \n    Limit: skip=None, fetch=1010\
+        \n    Limit: skip=0, fetch=1010\
         \n      TableScan: test, fetch=1010";
 
         assert_optimized_plan_eq(&plan, expected);
@@ -618,12 +672,12 @@ mod test {
                 (vec!["a"], vec!["a"]),
                 None,
             )?
-            .limit(Some(10), Some(1000))?
+            .limit(10, Some(1000))?
             .build()?;
 
         // Limit pushdown Not supported in Join
         let expected = "Limit: skip=10, fetch=1000\
-        \n  Inner Join: #test.a = #test2.a\
+        \n  Inner Join: test.a = test2.a\
         \n    TableScan: test\
         \n    TableScan: test2";
 
@@ -644,12 +698,12 @@ mod test {
                 (vec!["a"], vec!["a"]),
                 None,
             )?
-            .limit(Some(10), Some(1000))?
+            .limit(10, Some(1000))?
             .build()?;
 
         // Limit pushdown Not supported in Join
         let expected = "Limit: skip=10, fetch=1000\
-        \n  Inner Join: #test.a = #test2.a\
+        \n  Inner Join: test.a = test2.a\
         \n    TableScan: test\
         \n    TableScan: test2";
 
@@ -671,17 +725,17 @@ mod test {
         let outer_query = LogicalPlanBuilder::from(table_scan_2)
             .project(vec![col("a")])?
             .filter(exists(Arc::new(subquery)))?
-            .limit(Some(10), Some(100))?
+            .limit(10, Some(100))?
             .build()?;
 
         // Limit pushdown Not supported in sub_query
         let expected = "Limit: skip=10, fetch=100\
         \n  Filter: EXISTS (<subquery>)\
         \n    Subquery:\
-        \n      Filter: #test1.a = #test1.a\
-        \n        Projection: #test1.a\
+        \n      Filter: test1.a = test1.a\
+        \n        Projection: test1.a\
         \n          TableScan: test1\
-        \n    Projection: #test2.a\
+        \n    Projection: test2.a\
         \n      TableScan: test2";
 
         assert_optimized_plan_eq(&outer_query, expected);
@@ -702,17 +756,17 @@ mod test {
         let outer_query = LogicalPlanBuilder::from(table_scan_2)
             .project(vec![col("a")])?
             .filter(exists(Arc::new(subquery)))?
-            .limit(Some(10), Some(100))?
+            .limit(10, Some(100))?
             .build()?;
 
         // Limit pushdown Not supported in sub_query
         let expected = "Limit: skip=10, fetch=100\
         \n  Filter: EXISTS (<subquery>)\
         \n    Subquery:\
-        \n      Filter: #test1.a = #test1.a\
-        \n        Projection: #test1.a\
+        \n      Filter: test1.a = test1.a\
+        \n        Projection: test1.a\
         \n          TableScan: test1\
-        \n    Projection: #test2.a\
+        \n    Projection: test2.a\
         \n      TableScan: test2";
 
         assert_optimized_plan_eq(&outer_query, expected);
@@ -732,12 +786,12 @@ mod test {
                 (vec!["a"], vec!["a"]),
                 None,
             )?
-            .limit(None, Some(1000))?
+            .limit(0, Some(1000))?
             .build()?;
 
         // Limit pushdown Not supported in Join
-        let expected = "Limit: skip=None, fetch=1000\
-        \n  Left Join: #test.a = #test2.a\
+        let expected = "Limit: skip=0, fetch=1000\
+        \n  Left Join: test.a = test2.a\
         \n    TableScan: test, fetch=1000\
         \n    TableScan: test2";
 
@@ -758,12 +812,12 @@ mod test {
                 (vec!["a"], vec!["a"]),
                 None,
             )?
-            .limit(Some(10), Some(1000))?
+            .limit(10, Some(1000))?
             .build()?;
 
         // Limit pushdown Not supported in Join
         let expected = "Limit: skip=10, fetch=1000\
-        \n  Left Join: #test.a = #test2.a\
+        \n  Left Join: test.a = test2.a\
         \n    TableScan: test, fetch=1010\
         \n    TableScan: test2";
 
@@ -784,12 +838,12 @@ mod test {
                 (vec!["a"], vec!["a"]),
                 None,
             )?
-            .limit(None, Some(1000))?
+            .limit(0, Some(1000))?
             .build()?;
 
         // Limit pushdown Not supported in Join
-        let expected = "Limit: skip=None, fetch=1000\
-        \n  Right Join: #test.a = #test2.a\
+        let expected = "Limit: skip=0, fetch=1000\
+        \n  Right Join: test.a = test2.a\
         \n    TableScan: test\
         \n    TableScan: test2, fetch=1000";
 
@@ -810,12 +864,12 @@ mod test {
                 (vec!["a"], vec!["a"]),
                 None,
             )?
-            .limit(Some(10), Some(1000))?
+            .limit(10, Some(1000))?
             .build()?;
 
         // Limit pushdown with offset supported in right outer join
         let expected = "Limit: skip=10, fetch=1000\
-        \n  Right Join: #test.a = #test2.a\
+        \n  Right Join: test.a = test2.a\
         \n    TableScan: test\
         \n    TableScan: test2, fetch=1010";
 
