@@ -17,9 +17,9 @@
 
 use crate::{OptimizerConfig, OptimizerRule};
 use datafusion_common::Result;
+use datafusion_expr::expr::BinaryExpr;
 use datafusion_expr::logical_plan::Filter;
 use datafusion_expr::utils::from_plan;
-use datafusion_expr::Expr::BinaryExpr;
 use datafusion_expr::{Expr, LogicalPlan, Operator};
 use std::sync::Arc;
 
@@ -122,32 +122,23 @@ impl RewriteDisjunctivePredicate {
     pub fn new() -> Self {
         Self::default()
     }
-    fn rewrite_disjunctive_predicate(
-        &self,
-        plan: &LogicalPlan,
-        _optimizer_config: &OptimizerConfig,
-    ) -> Result<LogicalPlan> {
+    fn rewrite_disjunctive_predicate(plan: &LogicalPlan) -> Result<LogicalPlan> {
         match plan {
             LogicalPlan::Filter(filter) => {
-                let predicate = predicate(&filter.predicate)?;
+                let predicate = predicate(filter.predicate())?;
                 let rewritten_predicate = rewrite_predicate(predicate);
                 let rewritten_expr = normalize_predicate(rewritten_predicate);
-                Ok(LogicalPlan::Filter(Filter {
-                    predicate: rewritten_expr,
-                    input: Arc::new(self.rewrite_disjunctive_predicate(
-                        &filter.input,
-                        _optimizer_config,
-                    )?),
-                }))
+                Ok(LogicalPlan::Filter(Filter::try_new(
+                    rewritten_expr,
+                    Arc::new(Self::rewrite_disjunctive_predicate(filter.input())?),
+                )?))
             }
             _ => {
                 let expr = plan.expressions();
                 let inputs = plan.inputs();
                 let new_inputs = inputs
                     .iter()
-                    .map(|input| {
-                        self.rewrite_disjunctive_predicate(input, _optimizer_config)
-                    })
+                    .map(|input| Self::rewrite_disjunctive_predicate(input))
                     .collect::<Result<Vec<_>>>()?;
                 from_plan(plan, &expr, &new_inputs)
             }
@@ -159,9 +150,9 @@ impl OptimizerRule for RewriteDisjunctivePredicate {
     fn optimize(
         &self,
         plan: &LogicalPlan,
-        optimizer_config: &mut OptimizerConfig,
+        _optimizer_config: &mut OptimizerConfig,
     ) -> Result<LogicalPlan> {
-        self.rewrite_disjunctive_predicate(plan, optimizer_config)
+        Self::rewrite_disjunctive_predicate(plan)
     }
 
     fn name(&self) -> &str {
@@ -178,7 +169,7 @@ enum Predicate {
 
 fn predicate(expr: &Expr) -> Result<Predicate> {
     match expr {
-        BinaryExpr { left, op, right } => match op {
+        Expr::BinaryExpr(BinaryExpr { left, op, right }) => match op {
             Operator::And => {
                 let args = vec![predicate(left)?, predicate(right)?];
                 Ok(Predicate::And { args })
@@ -188,11 +179,11 @@ fn predicate(expr: &Expr) -> Result<Predicate> {
                 Ok(Predicate::Or { args })
             }
             _ => Ok(Predicate::Other {
-                expr: Box::new(BinaryExpr {
-                    left: left.clone(),
-                    op: *op,
-                    right: right.clone(),
-                }),
+                expr: Box::new(Expr::BinaryExpr(BinaryExpr::new(
+                    left.clone(),
+                    *op,
+                    right.clone(),
+                ))),
             }),
         },
         _ => Ok(Predicate::Other {
