@@ -17,20 +17,26 @@
 
 //! Optimizer rule for type validation and coercion
 
+use crate::utils::rewrite_preserving_name;
 use crate::{OptimizerConfig, OptimizerRule};
-use arrow::datatypes::DataType;
-use datafusion_common::{DFSchema, DFSchemaRef, DataFusionError, Result};
-use datafusion_expr::expr_rewriter::{ExprRewritable, ExprRewriter, RewriteRecursion};
+use arrow::datatypes::{DataType, IntervalUnit};
+use datafusion_common::{
+    parse_interval, DFSchema, DFSchemaRef, DataFusionError, Result, ScalarValue,
+};
+use datafusion_expr::expr::{Between, BinaryExpr, Case, Like};
+use datafusion_expr::expr_rewriter::{ExprRewriter, RewriteRecursion};
 use datafusion_expr::logical_plan::Subquery;
 use datafusion_expr::type_coercion::binary::{coerce_types, comparison_coercion};
 use datafusion_expr::type_coercion::functions::data_types;
 use datafusion_expr::type_coercion::other::{
     get_coerce_type_for_case_when, get_coerce_type_for_list,
 };
+use datafusion_expr::type_coercion::{is_date, is_numeric, is_timestamp};
 use datafusion_expr::utils::from_plan;
 use datafusion_expr::{
-    is_false, is_not_false, is_not_true, is_not_unknown, is_true, is_unknown, Expr,
-    LogicalPlan, Operator,
+    aggregate_function, function, is_false, is_not_false, is_not_true, is_not_unknown,
+    is_true, is_unknown, type_coercion, AggregateFunction, Expr, LogicalPlan, Operator,
+    WindowFrame, WindowFrameBound, WindowFrameUnits,
 };
 use datafusion_expr::{ExprSchemable, Signature};
 use std::sync::Arc;
@@ -52,9 +58,9 @@ impl OptimizerRule for TypeCoercion {
     fn optimize(
         &self,
         plan: &LogicalPlan,
-        optimizer_config: &mut OptimizerConfig,
+        _optimizer_config: &mut OptimizerConfig,
     ) -> Result<LogicalPlan> {
-        optimize_internal(&DFSchema::empty(), plan, optimizer_config)
+        optimize_internal(&DFSchema::empty(), plan)
     }
 }
 
@@ -62,15 +68,13 @@ fn optimize_internal(
     // use the external schema to handle the correlated subqueries case
     external_schema: &DFSchema,
     plan: &LogicalPlan,
-    optimizer_config: &mut OptimizerConfig,
 ) -> Result<LogicalPlan> {
     // optimize child plans first
     let new_inputs = plan
         .inputs()
         .iter()
-        .map(|p| optimize_internal(external_schema, p, optimizer_config))
+        .map(|p| optimize_internal(external_schema, p))
         .collect::<Result<Vec<_>>>()?;
-
     // get schema representing all available input fields. This is used for data type
     // resolution only, so order does not matter here
     let mut schema = new_inputs.iter().map(|input| input.schema()).fold(
@@ -90,37 +94,20 @@ fn optimize_internal(
         schema: Arc::new(schema),
     };
 
-    let original_expr_names: Vec<Option<String>> = plan
-        .expressions()
-        .iter()
-        .map(|expr| expr.name().ok())
-        .collect();
-
     let new_expr = plan
         .expressions()
         .into_iter()
-        .zip(original_expr_names)
-        .map(|(expr, original_name)| {
-            let expr = expr.rewrite(&mut expr_rewrite)?;
-
+        .map(|expr| {
             // ensure aggregate names don't change:
             // https://github.com/apache/arrow-datafusion/issues/3555
-            if matches!(expr, Expr::AggregateFunction { .. }) {
-                if let Some((alias, name)) = original_name.zip(expr.name().ok()) {
-                    if alias != name {
-                        return Ok(expr.alias(&alias));
-                    }
-                }
-            }
-
-            Ok(expr)
+            rewrite_preserving_name(expr, &mut expr_rewrite)
         })
         .collect::<Result<Vec<_>>>()?;
 
     from_plan(plan, &new_expr, &new_inputs)
 }
 
-struct TypeCoercionRewriter {
+pub(crate) struct TypeCoercionRewriter {
     pub(crate) schema: DFSchemaRef,
 }
 
@@ -132,18 +119,11 @@ impl ExprRewriter for TypeCoercionRewriter {
     fn mutate(&mut self, expr: Expr) -> Result<Expr> {
         match expr {
             Expr::ScalarSubquery(Subquery { subquery }) => {
-                let mut optimizer_config = OptimizerConfig::new();
-                let new_plan =
-                    optimize_internal(&self.schema, &subquery, &mut optimizer_config)?;
+                let new_plan = optimize_internal(&self.schema, &subquery)?;
                 Ok(Expr::ScalarSubquery(Subquery::new(new_plan)))
             }
             Expr::Exists { subquery, negated } => {
-                let mut optimizer_config = OptimizerConfig::new();
-                let new_plan = optimize_internal(
-                    &self.schema,
-                    &subquery.subquery,
-                    &mut optimizer_config,
-                )?;
+                let new_plan = optimize_internal(&self.schema, &subquery.subquery)?;
                 Ok(Expr::Exists {
                     subquery: Subquery::new(new_plan),
                     negated,
@@ -154,12 +134,7 @@ impl ExprRewriter for TypeCoercionRewriter {
                 subquery,
                 negated,
             } => {
-                let mut optimizer_config = OptimizerConfig::new();
-                let new_plan = optimize_internal(
-                    &self.schema,
-                    &subquery.subquery,
-                    &mut optimizer_config,
-                )?;
+                let new_plan = optimize_internal(&self.schema, &subquery.subquery)?;
                 Ok(Expr::InSubquery {
                     expr,
                     subquery: Subquery::new(new_plan),
@@ -183,44 +158,34 @@ impl ExprRewriter for TypeCoercionRewriter {
                     is_not_false(get_casted_expr_for_bool_op(&expr, &self.schema)?);
                 Ok(expr)
             }
-            Expr::Like {
+            Expr::Like(Like {
                 negated,
                 expr,
                 pattern,
                 escape_char,
-            } => {
+            }) => {
                 let left_type = expr.get_type(&self.schema)?;
                 let right_type = pattern.get_type(&self.schema)?;
                 let coerced_type =
                     coerce_types(&left_type, &Operator::Like, &right_type)?;
                 let expr = Box::new(expr.cast_to(&coerced_type, &self.schema)?);
                 let pattern = Box::new(pattern.cast_to(&coerced_type, &self.schema)?);
-                let expr = Expr::Like {
-                    negated,
-                    expr,
-                    pattern,
-                    escape_char,
-                };
+                let expr = Expr::Like(Like::new(negated, expr, pattern, escape_char));
                 Ok(expr)
             }
-            Expr::ILike {
+            Expr::ILike(Like {
                 negated,
                 expr,
                 pattern,
                 escape_char,
-            } => {
+            }) => {
                 let left_type = expr.get_type(&self.schema)?;
                 let right_type = pattern.get_type(&self.schema)?;
                 let coerced_type =
                     coerce_types(&left_type, &Operator::Like, &right_type)?;
                 let expr = Box::new(expr.cast_to(&coerced_type, &self.schema)?);
                 let pattern = Box::new(pattern.cast_to(&coerced_type, &self.schema)?);
-                let expr = Expr::ILike {
-                    negated,
-                    expr,
-                    pattern,
-                    escape_char,
-                };
+                let expr = Expr::ILike(Like::new(negated, expr, pattern, escape_char));
                 Ok(expr)
             }
             Expr::IsUnknown(expr) => {
@@ -241,11 +206,11 @@ impl ExprRewriter for TypeCoercionRewriter {
                 let expr = is_not_unknown(expr.cast_to(&coerced_type, &self.schema)?);
                 Ok(expr)
             }
-            Expr::BinaryExpr {
+            Expr::BinaryExpr(BinaryExpr {
                 ref left,
                 op,
                 ref right,
-            } => {
+            }) => {
                 let left_type = left.get_type(&self.schema)?;
                 let right_type = right.get_type(&self.schema)?;
                 match (&left_type, &right_type) {
@@ -258,25 +223,21 @@ impl ExprRewriter for TypeCoercionRewriter {
                     }
                     _ => {
                         let coerced_type = coerce_types(&left_type, &op, &right_type)?;
-                        let expr = Expr::BinaryExpr {
-                            left: Box::new(
-                                left.clone().cast_to(&coerced_type, &self.schema)?,
-                            ),
+                        let expr = Expr::BinaryExpr(BinaryExpr::new(
+                            Box::new(left.clone().cast_to(&coerced_type, &self.schema)?),
                             op,
-                            right: Box::new(
-                                right.clone().cast_to(&coerced_type, &self.schema)?,
-                            ),
-                        };
+                            Box::new(right.clone().cast_to(&coerced_type, &self.schema)?),
+                        ));
                         Ok(expr)
                     }
                 }
             }
-            Expr::Between {
+            Expr::Between(Between {
                 expr,
                 negated,
                 low,
                 high,
-            } => {
+            }) => {
                 let expr_type = expr.get_type(&self.schema)?;
                 let low_type = low.get_type(&self.schema)?;
                 let low_coerced_type = comparison_coercion(&expr_type, &low_type)
@@ -302,24 +263,12 @@ impl ExprRewriter for TypeCoercionRewriter {
                                 expr_type, high_type
                             ))
                         })?;
-                let expr = Expr::Between {
-                    expr: Box::new(expr.cast_to(&coercion_type, &self.schema)?),
+                let expr = Expr::Between(Between::new(
+                    Box::new(expr.cast_to(&coercion_type, &self.schema)?),
                     negated,
-                    low: Box::new(low.cast_to(&coercion_type, &self.schema)?),
-                    high: Box::new(high.cast_to(&coercion_type, &self.schema)?),
-                };
-                Ok(expr)
-            }
-            Expr::ScalarUDF { fun, args } => {
-                let new_expr = coerce_arguments_for_signature(
-                    args.as_slice(),
-                    &self.schema,
-                    &fun.signature,
-                )?;
-                let expr = Expr::ScalarUDF {
-                    fun,
-                    args: new_expr,
-                };
+                    Box::new(low.cast_to(&coercion_type, &self.schema)?),
+                    Box::new(high.cast_to(&coercion_type, &self.schema)?),
+                ));
                 Ok(expr)
             }
             Expr::InList {
@@ -357,18 +306,15 @@ impl ExprRewriter for TypeCoercionRewriter {
                     }
                 }
             }
-            Expr::Case {
-                expr,
-                when_then_expr,
-                else_expr,
-            } => {
+            Expr::Case(case) => {
                 // all the result of then and else should be convert to a common data type,
                 // if they can be coercible to a common data type, return error.
-                let then_types = when_then_expr
+                let then_types = case
+                    .when_then_expr
                     .iter()
                     .map(|when_then| when_then.1.get_type(&self.schema))
                     .collect::<Result<Vec<_>>>()?;
-                let else_type = match &else_expr {
+                let else_type = match &case.else_expr {
                     None => Ok(None),
                     Some(expr) => expr.get_type(&self.schema).map(Some),
                 }?;
@@ -380,32 +326,195 @@ impl ExprRewriter for TypeCoercionRewriter {
                         then_types, else_type
                     ))),
                     Some(data_type) => {
-                        let left = when_then_expr
+                        let left = case.when_then_expr
                             .into_iter()
                             .map(|(when, then)| {
                                 let then = then.cast_to(&data_type, &self.schema)?;
                                 Ok((when, Box::new(then)))
                             })
                             .collect::<Result<Vec<_>>>()?;
-                        let right = match else_expr {
+                        let right = match &case.else_expr {
                             None => None,
                             Some(expr) => {
-                                Some(Box::new(expr.cast_to(&data_type, &self.schema)?))
+                                Some(Box::new(expr.clone().cast_to(&data_type, &self.schema)?))
                             }
                         };
-                        Ok(Expr::Case {
-                            expr,
-                            when_then_expr: left,
-                            else_expr: right,
-                        })
+                        Ok(Expr::Case(Case::new(case.expr,left,right)))
                     }
                 }
             }
+            Expr::ScalarUDF { fun, args } => {
+                let new_expr = coerce_arguments_for_signature(
+                    args.as_slice(),
+                    &self.schema,
+                    &fun.signature,
+                )?;
+                let expr = Expr::ScalarUDF {
+                    fun,
+                    args: new_expr,
+                };
+                Ok(expr)
+            }
+            Expr::ScalarFunction { fun, args } => {
+                let nex_expr = coerce_arguments_for_signature(
+                    args.as_slice(),
+                    &self.schema,
+                    &function::signature(&fun),
+                )?;
+                let expr = Expr::ScalarFunction {
+                    fun,
+                    args: nex_expr,
+                };
+                Ok(expr)
+            }
+            Expr::AggregateFunction {
+                fun,
+                args,
+                distinct,
+                filter,
+            } => {
+                let new_expr = coerce_agg_exprs_for_signature(
+                    &fun,
+                    &args,
+                    &self.schema,
+                    &aggregate_function::signature(&fun),
+                )?;
+                let expr = Expr::AggregateFunction {
+                    fun,
+                    args: new_expr,
+                    distinct,
+                    filter,
+                };
+                Ok(expr)
+            }
+            Expr::AggregateUDF { fun, args, filter } => {
+                let new_expr = coerce_arguments_for_signature(
+                    args.as_slice(),
+                    &self.schema,
+                    &fun.signature,
+                )?;
+                let expr = Expr::AggregateUDF {
+                    fun,
+                    args: new_expr,
+                    filter,
+                };
+                Ok(expr)
+            }
+            Expr::WindowFunction {
+                fun,
+                args,
+                partition_by,
+                order_by,
+                window_frame,
+            } => {
+                let window_frame =
+                    get_coerced_window_frame(window_frame, &self.schema, &order_by)?;
+                let expr = Expr::WindowFunction {
+                    fun,
+                    args,
+                    partition_by,
+                    order_by,
+                    window_frame,
+                };
+                Ok(expr)
+            }
             expr => Ok(expr),
         }
     }
 }
 
+/// Casts the ScalarValue `value` to coerced type.
+// When coerced type is `Interval` we use `parse_interval` since `try_from_string` not
+// supports conversion from string to Interval
+fn convert_to_coerced_type(
+    coerced_type: &DataType,
+    value: &ScalarValue,
+) -> Result<ScalarValue> {
+    match value {
+        // In here we do casting either for ScalarValue::Utf8(None) or
+        // ScalarValue::Utf8(Some(val)). The other types are already casted.
+        // The reason is that we convert the sqlparser result
+        // to the Utf8 for all possible cases. Hence the types other than Utf8
+        // are already casted to appropriate type. Therefore they can be returned directly.
+        ScalarValue::Utf8(None) => ScalarValue::try_from(coerced_type),
+        ScalarValue::Utf8(Some(val)) => {
+            // we need special handling for Interval types
+            if let DataType::Interval(..) = coerced_type {
+                parse_interval("millisecond", val)
+            } else {
+                ScalarValue::try_from_string(val.clone(), coerced_type)
+            }
+        }
+        s => Ok(s.clone()),
+    }
+}
+
+fn coerce_frame_bound(
+    coerced_type: &DataType,
+    bound: &WindowFrameBound,
+) -> Result<WindowFrameBound> {
+    Ok(match bound {
+        WindowFrameBound::Preceding(val) => {
+            WindowFrameBound::Preceding(convert_to_coerced_type(coerced_type, val)?)
+        }
+        WindowFrameBound::CurrentRow => WindowFrameBound::CurrentRow,
+        WindowFrameBound::Following(val) => {
+            WindowFrameBound::Following(convert_to_coerced_type(coerced_type, val)?)
+        }
+    })
+}
+
+fn get_coerced_window_frame(
+    window_frame: Option<WindowFrame>,
+    schema: &DFSchemaRef,
+    expressions: &[Expr],
+) -> Result<Option<WindowFrame>> {
+    fn get_coerced_type(column_type: &DataType) -> Result<DataType> {
+        if is_numeric(column_type) {
+            Ok(column_type.clone())
+        } else if is_timestamp(column_type) || is_date(column_type) {
+            Ok(DataType::Interval(IntervalUnit::MonthDayNano))
+        } else {
+            Err(DataFusionError::Internal(format!(
+                "Cannot run range queries on datatype: {:?}",
+                column_type
+            )))
+        }
+    }
+
+    if let Some(window_frame) = window_frame {
+        let mut window_frame = window_frame;
+        let current_types = expressions
+            .iter()
+            .map(|e| e.get_type(schema))
+            .collect::<Result<Vec<_>>>()?;
+        match &mut window_frame.units {
+            WindowFrameUnits::Range => {
+                let col_type = current_types.first().ok_or_else(|| {
+                    DataFusionError::Internal(
+                        "ORDER BY column cannot be empty".to_string(),
+                    )
+                })?;
+                let coerced_type = get_coerced_type(col_type)?;
+                window_frame.start_bound =
+                    coerce_frame_bound(&coerced_type, &window_frame.start_bound)?;
+                window_frame.end_bound =
+                    coerce_frame_bound(&coerced_type, &window_frame.end_bound)?;
+            }
+            WindowFrameUnits::Rows | WindowFrameUnits::Groups => {
+                let coerced_type = DataType::UInt64;
+                window_frame.start_bound =
+                    coerce_frame_bound(&coerced_type, &window_frame.start_bound)?;
+                window_frame.end_bound =
+                    coerce_frame_bound(&coerced_type, &window_frame.end_bound)?;
+            }
+        }
+
+        Ok(Some(window_frame))
+    } else {
+        Ok(None)
+    }
+}
 // Support the `IsTrue` `IsNotTrue` `IsFalse` `IsNotFalse` type coercion.
 // The above op will be rewrite to the binary op when creating the physical op.
 fn get_casted_expr_for_bool_op(expr: &Expr, schema: &DFSchemaRef) -> Result<Expr> {
@@ -442,20 +551,53 @@ fn coerce_arguments_for_signature(
         .collect::<Result<Vec<_>>>()
 }
 
+/// Returns the coerced exprs for each `input_exprs`.
+/// Get the coerced data type from `aggregate_rule::coerce_types` and add `try_cast` if the
+/// data type of `input_exprs` need to be coerced.
+fn coerce_agg_exprs_for_signature(
+    agg_fun: &AggregateFunction,
+    input_exprs: &[Expr],
+    schema: &DFSchema,
+    signature: &Signature,
+) -> Result<Vec<Expr>> {
+    if input_exprs.is_empty() {
+        return Ok(vec![]);
+    }
+    let current_types = input_exprs
+        .iter()
+        .map(|e| e.get_type(schema))
+        .collect::<Result<Vec<_>>>()?;
+
+    let coerced_types =
+        type_coercion::aggregates::coerce_types(agg_fun, &current_types, signature)?;
+
+    input_exprs
+        .iter()
+        .enumerate()
+        .map(|(i, expr)| expr.clone().cast_to(&coerced_types[i], schema))
+        .collect::<Result<Vec<_>>>()
+}
+
 #[cfg(test)]
 mod test {
     use crate::type_coercion::{TypeCoercion, TypeCoercionRewriter};
     use crate::{OptimizerConfig, OptimizerRule};
     use arrow::datatypes::DataType;
     use datafusion_common::{DFField, DFSchema, Result, ScalarValue};
+    use datafusion_expr::expr::Like;
     use datafusion_expr::expr_rewriter::ExprRewritable;
-    use datafusion_expr::{cast, col, is_true, ColumnarValue};
+    use datafusion_expr::{
+        cast, col, concat, concat_ws, create_udaf, is_true,
+        AccumulatorFunctionImplementation, AggregateFunction, AggregateUDF,
+        BuiltinScalarFunction, ColumnarValue, StateTypeFunction,
+    };
     use datafusion_expr::{
         lit,
         logical_plan::{EmptyRelation, Projection},
         Expr, LogicalPlan, ReturnTypeFunction, ScalarFunctionImplementation, ScalarUDF,
         Signature, Volatility,
     };
+    use datafusion_physical_expr::expressions::AvgAccumulator;
     use std::sync::Arc;
 
     #[test]
@@ -564,6 +706,147 @@ mod test {
         Ok(())
     }
 
+    #[test]
+    fn scalar_function() -> Result<()> {
+        let empty = empty();
+        let lit_expr = lit(10i64);
+        let fun: BuiltinScalarFunction = BuiltinScalarFunction::Abs;
+        let scalar_function_expr = Expr::ScalarFunction {
+            fun,
+            args: vec![lit_expr],
+        };
+        let plan = LogicalPlan::Projection(Projection::try_new(
+            vec![scalar_function_expr],
+            empty,
+            None,
+        )?);
+        let rule = TypeCoercion::new();
+        let mut config = OptimizerConfig::default();
+        let plan = rule.optimize(&plan, &mut config)?;
+        assert_eq!(
+            "Projection: abs(CAST(Int64(10) AS Float64))\n  EmptyRelation",
+            &format!("{:?}", plan)
+        );
+        Ok(())
+    }
+
+    #[test]
+    fn agg_udaf() -> Result<()> {
+        let empty = empty();
+        let my_avg = create_udaf(
+            "MY_AVG",
+            DataType::Float64,
+            Arc::new(DataType::Float64),
+            Volatility::Immutable,
+            Arc::new(|_| Ok(Box::new(AvgAccumulator::try_new(&DataType::Float64)?))),
+            Arc::new(vec![DataType::UInt64, DataType::Float64]),
+        );
+        let udaf = Expr::AggregateUDF {
+            fun: Arc::new(my_avg),
+            args: vec![lit(10i64)],
+            filter: None,
+        };
+        let plan = LogicalPlan::Projection(Projection::try_new(vec![udaf], empty, None)?);
+        let rule = TypeCoercion::new();
+        let mut config = OptimizerConfig::default();
+        let plan = rule.optimize(&plan, &mut config)?;
+        assert_eq!(
+            "Projection: MY_AVG(CAST(Int64(10) AS Float64))\n  EmptyRelation",
+            &format!("{:?}", plan)
+        );
+        Ok(())
+    }
+
+    #[test]
+    fn agg_udaf_invalid_input() -> Result<()> {
+        let empty = empty();
+        let return_type: ReturnTypeFunction =
+            Arc::new(move |_| Ok(Arc::new(DataType::Float64)));
+        let state_type: StateTypeFunction =
+            Arc::new(move |_| Ok(Arc::new(vec![DataType::UInt64, DataType::Float64])));
+        let accumulator: AccumulatorFunctionImplementation =
+            Arc::new(|_| Ok(Box::new(AvgAccumulator::try_new(&DataType::Float64)?)));
+        let my_avg = AggregateUDF::new(
+            "MY_AVG",
+            &Signature::uniform(1, vec![DataType::Float64], Volatility::Immutable),
+            &return_type,
+            &accumulator,
+            &state_type,
+        );
+        let udaf = Expr::AggregateUDF {
+            fun: Arc::new(my_avg),
+            args: vec![lit("10")],
+            filter: None,
+        };
+        let plan = LogicalPlan::Projection(Projection::try_new(vec![udaf], empty, None)?);
+        let rule = TypeCoercion::new();
+        let mut config = OptimizerConfig::default();
+        let plan = rule.optimize(&plan, &mut config);
+        assert!(plan.is_err());
+        assert_eq!(
+            "Plan(\"Coercion from [Utf8] to the signature Uniform(1, [Float64]) failed.\")",
+            &format!("{:?}", plan.err().unwrap())
+        );
+        Ok(())
+    }
+
+    #[test]
+    fn agg_function_case() -> Result<()> {
+        let empty = empty();
+        let fun: AggregateFunction = AggregateFunction::Avg;
+        let agg_expr = Expr::AggregateFunction {
+            fun,
+            args: vec![lit(12i64)],
+            distinct: false,
+            filter: None,
+        };
+        let plan =
+            LogicalPlan::Projection(Projection::try_new(vec![agg_expr], empty, None)?);
+        let rule = TypeCoercion::new();
+        let mut config = OptimizerConfig::default();
+        let plan = rule.optimize(&plan, &mut config)?;
+        assert_eq!(
+            "Projection: AVG(Int64(12))\n  EmptyRelation",
+            &format!("{:?}", plan)
+        );
+
+        let empty = empty_with_type(DataType::Int32);
+        let fun: AggregateFunction = AggregateFunction::Avg;
+        let agg_expr = Expr::AggregateFunction {
+            fun,
+            args: vec![col("a")],
+            distinct: false,
+            filter: None,
+        };
+        let plan =
+            LogicalPlan::Projection(Projection::try_new(vec![agg_expr], empty, None)?);
+        let plan = rule.optimize(&plan, &mut config)?;
+        assert_eq!(
+            "Projection: AVG(a)\n  EmptyRelation",
+            &format!("{:?}", plan)
+        );
+        Ok(())
+    }
+
+    #[test]
+    fn agg_function_invalid_input() -> Result<()> {
+        let empty = empty();
+        let fun: AggregateFunction = AggregateFunction::Avg;
+        let agg_expr = Expr::AggregateFunction {
+            fun,
+            args: vec![lit("1")],
+            distinct: false,
+            filter: None,
+        };
+        let expr = Projection::try_new(vec![agg_expr], empty, None);
+        assert!(expr.is_err());
+        assert_eq!(
+            "Plan(\"The function Avg does not support inputs of type Utf8.\")",
+            &format!("{:?}", expr.err().unwrap())
+        );
+        Ok(())
+    }
+
     #[test]
     fn binary_op_date32_add_interval() -> Result<()> {
         //CAST(Utf8("1998-03-18") AS Date32) + IntervalDayTime("386547056640")
@@ -603,7 +886,8 @@ mod test {
         let mut config = OptimizerConfig::default();
         let plan = rule.optimize(&plan, &mut config)?;
         assert_eq!(
-            "Projection: a IN ([CAST(Int32(1) AS Int64), CAST(Int8(4) AS Int64), Int64(8)])\n  EmptyRelation",
+            "Projection: a IN ([CAST(Int32(1) AS Int64), CAST(Int8(4) AS Int64), Int64(8)]) AS a IN (Map { iter: Iter([Int32(1), Int8(4), Int64(8)]) })\
+             \n  EmptyRelation",
             &format!("{:?}", plan)
         );
         // a in (1,4,8), a is decimal
@@ -621,7 +905,8 @@ mod test {
         let plan = LogicalPlan::Projection(Projection::try_new(vec![expr], empty, None)?);
         let plan = rule.optimize(&plan, &mut config)?;
         assert_eq!(
-            "Projection: CAST(a AS Decimal128(24, 4)) IN ([CAST(Int32(1) AS Decimal128(24, 4)), CAST(Int8(4) AS Decimal128(24, 4)), CAST(Int64(8) AS Decimal128(24, 4))])\n  EmptyRelation",
+            "Projection: CAST(a AS Decimal128(24, 4)) IN ([CAST(Int32(1) AS Decimal128(24, 4)), CAST(Int8(4) AS Decimal128(24, 4)), CAST(Int64(8) AS Decimal128(24, 4))]) AS a IN (Map { iter: Iter([Int32(1), Int8(4), Int64(8)]) })\
+             \n  EmptyRelation",
             &format!("{:?}", plan)
         );
         Ok(())
@@ -687,12 +972,7 @@ mod test {
         // like : utf8 like "abc"
         let expr = Box::new(col("a"));
         let pattern = Box::new(lit(ScalarValue::new_utf8("abc")));
-        let like_expr = Expr::Like {
-            negated: false,
-            expr,
-            pattern,
-            escape_char: None,
-        };
+        let like_expr = Expr::Like(Like::new(false, expr, pattern, None));
         let empty = empty_with_type(DataType::Utf8);
         let plan =
             LogicalPlan::Projection(Projection::try_new(vec![like_expr], empty, None)?);
@@ -706,12 +986,7 @@ mod test {
 
         let expr = Box::new(col("a"));
         let pattern = Box::new(lit(ScalarValue::Null));
-        let like_expr = Expr::Like {
-            negated: false,
-            expr,
-            pattern,
-            escape_char: None,
-        };
+        let like_expr = Expr::Like(Like::new(false, expr, pattern, None));
         let empty = empty_with_type(DataType::Utf8);
         let plan =
             LogicalPlan::Projection(Projection::try_new(vec![like_expr], empty, None)?);
@@ -719,18 +994,14 @@ mod test {
         let mut config = OptimizerConfig::default();
         let plan = rule.optimize(&plan, &mut config).unwrap();
         assert_eq!(
-            "Projection: a LIKE CAST(NULL AS Utf8)\n  EmptyRelation",
+            "Projection: a LIKE CAST(NULL AS Utf8) AS a LIKE NULL \
+             \n  EmptyRelation",
             &format!("{:?}", plan)
         );
 
         let expr = Box::new(col("a"));
         let pattern = Box::new(lit(ScalarValue::new_utf8("abc")));
-        let like_expr = Expr::Like {
-            negated: false,
-            expr,
-            pattern,
-            escape_char: None,
-        };
+        let like_expr = Expr::Like(Like::new(false, expr, pattern, None));
         let empty = empty_with_type(DataType::Int64);
         let plan =
             LogicalPlan::Projection(Projection::try_new(vec![like_expr], empty, None)?);
@@ -782,6 +1053,47 @@ mod test {
         Ok(())
     }
 
+    #[test]
+    fn concat_for_type_coercion() -> Result<()> {
+        let empty = empty_with_type(DataType::Utf8);
+        let args = [col("a"), lit("b"), lit(true), lit(false), lit(13)];
+
+        // concat
+        {
+            let expr = concat(&args);
+
+            let plan = LogicalPlan::Projection(Projection::try_new(
+                vec![expr],
+                empty.clone(),
+                None,
+            )?);
+            let rule = TypeCoercion::new();
+            let mut config = OptimizerConfig::default();
+            let plan = rule.optimize(&plan, &mut config).unwrap();
+            assert_eq!(
+                "Projection: concat(a, Utf8(\"b\"), CAST(Boolean(true) AS Utf8), CAST(Boolean(false) AS Utf8), CAST(Int32(13) AS Utf8))\n  EmptyRelation",
+                &format!("{:?}", plan)
+            );
+        }
+
+        // concat_ws
+        {
+            let expr = concat_ws(lit("-"), args.to_vec());
+
+            let plan =
+                LogicalPlan::Projection(Projection::try_new(vec![expr], empty, None)?);
+            let rule = TypeCoercion::new();
+            let mut config = OptimizerConfig::default();
+            let plan = rule.optimize(&plan, &mut config).unwrap();
+            assert_eq!(
+                "Projection: concatwithseparator(Utf8(\"-\"), a, Utf8(\"b\"), CAST(Boolean(true) AS Utf8), CAST(Boolean(false) AS Utf8), CAST(Int32(13) AS Utf8))\n  EmptyRelation",
+                &format!("{:?}", plan)
+            );
+        }
+
+        Ok(())
+    }
+
     fn empty() -> Arc<LogicalPlan> {
         Arc::new(LogicalPlan::EmptyRelation(EmptyRelation {
             produce_one_row: false,
