@@ -31,7 +31,7 @@ use arrow::{
     array::{ArrayRef, UInt64Array},
     datatypes::Field,
 };
-use datafusion_common::ScalarValue;
+use datafusion_common::{downcast_value, ScalarValue};
 use datafusion_common::{DataFusionError, Result};
 use datafusion_expr::{Accumulator, AggregateState};
 use datafusion_row::accessor::RowAccessor;
@@ -161,23 +161,21 @@ impl Accumulator for AvgAccumulator {
         let values = &values[0];
 
         self.count += (values.len() - values.data().null_count()) as u64;
-        self.sum = sum::sum(
-            &self.sum,
-            &sum::sum_batch(values, &self.sum.get_datatype())?,
-        )?;
+        self.sum = self
+            .sum
+            .add(&sum::sum_batch(values, &self.sum.get_datatype())?)?;
         Ok(())
     }
 
     fn merge_batch(&mut self, states: &[ArrayRef]) -> Result<()> {
-        let counts = states[0].as_any().downcast_ref::<UInt64Array>().unwrap();
+        let counts = downcast_value!(states[0], UInt64Array);
         // counts are summed
         self.count += compute::sum(counts).unwrap_or(0);
 
         // sums are summed
-        self.sum = sum::sum(
-            &self.sum,
-            &sum::sum_batch(&states[1], &self.sum.get_datatype())?,
-        )?;
+        self.sum = self
+            .sum
+            .add(&sum::sum_batch(&states[1], &self.sum.get_datatype())?)?;
         Ok(())
     }
 
@@ -232,7 +230,6 @@ impl RowAccumulator for AvgRowAccumulator {
 
         // sum
         sum::add_to_row(
-            &self.sum_datatype,
             self.state_index() + 1,
             accessor,
             &sum::sum_batch(values, &self.sum_datatype)?,
@@ -245,18 +242,14 @@ impl RowAccumulator for AvgRowAccumulator {
         states: &[ArrayRef],
         accessor: &mut RowAccessor,
     ) -> Result<()> {
-        let counts = states[0].as_any().downcast_ref::<UInt64Array>().unwrap();
+        let counts = downcast_value!(states[0], UInt64Array);
         // count
         let delta = compute::sum(counts).unwrap_or(0);
         accessor.add_u64(self.state_index(), delta);
 
         // sum
-        sum::add_to_row(
-            &self.sum_datatype,
-            self.state_index() + 1,
-            accessor,
-            &sum::sum_batch(&states[1], &self.sum_datatype)?,
-        )?;
+        let difference = sum::sum_batch(&states[1], &self.sum_datatype)?;
+        sum::add_to_row(self.state_index() + 1, accessor, &difference)?;
         Ok(())
     }
 
@@ -303,8 +296,7 @@ mod tests {
             array,
             DataType::Decimal128(10, 0),
             Avg,
-            ScalarValue::Decimal128(Some(35000), 14, 4),
-            DataType::Decimal128(14, 4)
+            ScalarValue::Decimal128(Some(35000), 14, 4)
         )
     }
 
@@ -320,8 +312,7 @@ mod tests {
             array,
             DataType::Decimal128(10, 0),
             Avg,
-            ScalarValue::Decimal128(Some(32500), 14, 4),
-            DataType::Decimal128(14, 4)
+            ScalarValue::Decimal128(Some(32500), 14, 4)
         )
     }
 
@@ -329,8 +320,9 @@ mod tests {
     fn avg_decimal_all_nulls() -> Result<()> {
         // test agg
         let array: ArrayRef = Arc::new(
-            std::iter::repeat(None)
+            std::iter::repeat::<Option<i128>>(None)
                 .take(6)
+                .into_iter()
                 .collect::<Decimal128Array>()
                 .with_precision_and_scale(10, 0)?,
         );
@@ -338,21 +330,14 @@ mod tests {
             array,
             DataType::Decimal128(10, 0),
             Avg,
-            ScalarValue::Decimal128(None, 14, 4),
-            DataType::Decimal128(14, 4)
+            ScalarValue::Decimal128(None, 14, 4)
         )
     }
 
     #[test]
     fn avg_i32() -> Result<()> {
         let a: ArrayRef = Arc::new(Int32Array::from(vec![1, 2, 3, 4, 5]));
-        generic_test_op!(
-            a,
-            DataType::Int32,
-            Avg,
-            ScalarValue::from(3_f64),
-            DataType::Float64
-        )
+        generic_test_op!(a, DataType::Int32, Avg, ScalarValue::from(3_f64))
     }
 
     #[test]
@@ -364,63 +349,33 @@ mod tests {
             Some(4),
             Some(5),
         ]));
-        generic_test_op!(
-            a,
-            DataType::Int32,
-            Avg,
-            ScalarValue::from(3.25f64),
-            DataType::Float64
-        )
+        generic_test_op!(a, DataType::Int32, Avg, ScalarValue::from(3.25f64))
     }
 
     #[test]
     fn avg_i32_all_nulls() -> Result<()> {
         let a: ArrayRef = Arc::new(Int32Array::from(vec![None, None]));
-        generic_test_op!(
-            a,
-            DataType::Int32,
-            Avg,
-            ScalarValue::Float64(None),
-            DataType::Float64
-        )
+        generic_test_op!(a, DataType::Int32, Avg, ScalarValue::Float64(None))
     }
 
     #[test]
     fn avg_u32() -> Result<()> {
         let a: ArrayRef =
             Arc::new(UInt32Array::from(vec![1_u32, 2_u32, 3_u32, 4_u32, 5_u32]));
-        generic_test_op!(
-            a,
-            DataType::UInt32,
-            Avg,
-            ScalarValue::from(3.0f64),
-            DataType::Float64
-        )
+        generic_test_op!(a, DataType::UInt32, Avg, ScalarValue::from(3.0f64))
     }
 
     #[test]
     fn avg_f32() -> Result<()> {
         let a: ArrayRef =
             Arc::new(Float32Array::from(vec![1_f32, 2_f32, 3_f32, 4_f32, 5_f32]));
-        generic_test_op!(
-            a,
-            DataType::Float32,
-            Avg,
-            ScalarValue::from(3_f64),
-            DataType::Float64
-        )
+        generic_test_op!(a, DataType::Float32, Avg, ScalarValue::from(3_f64))
     }
 
     #[test]
     fn avg_f64() -> Result<()> {
         let a: ArrayRef =
             Arc::new(Float64Array::from(vec![1_f64, 2_f64, 3_f64, 4_f64, 5_f64]));
-        generic_test_op!(
-            a,
-            DataType::Float64,
-            Avg,
-            ScalarValue::from(3_f64),
-            DataType::Float64
-        )
+        generic_test_op!(a, DataType::Float64, Avg, ScalarValue::from(3_f64))
     }
 }
