@@ -19,7 +19,8 @@
 
 use arrow::{
     array::{
-        Array, ArrayRef, BinaryArray, GenericStringArray, OffsetSizeTrait, StringArray,
+        Array, ArrayRef, BinaryArray, GenericBinaryArray, GenericStringArray,
+        OffsetSizeTrait, StringArray,
     },
     datatypes::DataType,
 };
@@ -59,17 +60,22 @@ fn digest_process(
 ) -> Result<ColumnarValue> {
     match value {
         ColumnarValue::Array(a) => match a.data_type() {
-            DataType::Utf8 => digest_algorithm.digest_array::<i32>(a.as_ref()),
-            DataType::LargeUtf8 => digest_algorithm.digest_array::<i64>(a.as_ref()),
+            DataType::Utf8 => digest_algorithm.digest_utf8_array::<i32>(a.as_ref()),
+            DataType::LargeUtf8 => digest_algorithm.digest_utf8_array::<i64>(a.as_ref()),
+            DataType::Binary => digest_algorithm.digest_binary_array::<i32>(a.as_ref()),
+            DataType::LargeBinary => {
+                digest_algorithm.digest_binary_array::<i64>(a.as_ref())
+            }
             other => Err(DataFusionError::Internal(format!(
                 "Unsupported data type {:?} for function {}",
                 other, digest_algorithm,
             ))),
         },
         ColumnarValue::Scalar(scalar) => match scalar {
-            ScalarValue::Utf8(a) | ScalarValue::LargeUtf8(a) => {
-                Ok(digest_algorithm.digest_scalar(a))
-            }
+            ScalarValue::Utf8(a) | ScalarValue::LargeUtf8(a) => Ok(digest_algorithm
+                .digest_scalar(&a.as_ref().map(|s: &String| s.as_bytes()))),
+            ScalarValue::Binary(a) | ScalarValue::LargeBinary(a) => Ok(digest_algorithm
+                .digest_scalar(&a.as_ref().map(|v: &Vec<u8>| v.as_slice()))),
             other => Err(DataFusionError::Internal(format!(
                 "Unsupported data type {:?} for function {}",
                 other, digest_algorithm,
@@ -106,7 +112,7 @@ macro_rules! digest_to_scalar {
 
 impl DigestAlgorithm {
     /// digest an optional string to its hash value, null values are returned as is
-    fn digest_scalar(self, value: &Option<String>) -> ColumnarValue {
+    fn digest_scalar(self, value: &Option<&[u8]>) -> ColumnarValue {
         ColumnarValue::Scalar(match self {
             Self::Md5 => digest_to_scalar!(Md5, value),
             Self::Sha224 => digest_to_scalar!(Sha224, value),
@@ -115,16 +121,55 @@ impl DigestAlgorithm {
             Self::Sha512 => digest_to_scalar!(Sha512, value),
             Self::Blake2b => digest_to_scalar!(Blake2b512, value),
             Self::Blake2s => digest_to_scalar!(Blake2s256, value),
-            Self::Blake3 => ScalarValue::Binary(value.as_ref().map(|v| {
+            Self::Blake3 => ScalarValue::Binary(value.map(|v| {
                 let mut digest = Blake3::default();
-                digest.update(v.as_bytes());
+                digest.update(v);
                 digest.finalize().as_bytes().to_vec()
             })),
         })
     }
 
+    /// digest a binary array to their hash values
+    fn digest_binary_array<T>(self, value: &dyn Array) -> Result<ColumnarValue>
+    where
+        T: OffsetSizeTrait,
+    {
+        let input_value = value
+            .as_any()
+            .downcast_ref::<GenericBinaryArray<T>>()
+            .ok_or_else(|| {
+                DataFusionError::Internal(format!(
+                    "could not cast value to {}",
+                    type_name::<GenericBinaryArray<T>>()
+                ))
+            })?;
+        let array: ArrayRef = match self {
+            Self::Md5 => digest_to_array!(Md5, input_value),
+            Self::Sha224 => digest_to_array!(Sha224, input_value),
+            Self::Sha256 => digest_to_array!(Sha256, input_value),
+            Self::Sha384 => digest_to_array!(Sha384, input_value),
+            Self::Sha512 => digest_to_array!(Sha512, input_value),
+            Self::Blake2b => digest_to_array!(Blake2b512, input_value),
+            Self::Blake2s => digest_to_array!(Blake2s256, input_value),
+            Self::Blake3 => {
+                let binary_array: BinaryArray = input_value
+                    .iter()
+                    .map(|opt| {
+                        opt.map(|x| {
+                            let mut digest = Blake3::default();
+                            digest.update(x);
+                            digest.finalize().as_bytes().to_vec()
+                        })
+                    })
+                    .collect();
+                Arc::new(binary_array)
+            }
+        };
+        Ok(ColumnarValue::Array(array))
+    }
+
     /// digest a string array to their hash values
-    fn digest_array<T>(self, value: &dyn Array) -> Result<ColumnarValue>
+    fn digest_utf8_array<T>(self, value: &dyn Array) -> Result<ColumnarValue>
     where
         T: OffsetSizeTrait,
     {
