@@ -133,7 +133,7 @@ where
 {
     Ok(left
         .iter()
-        .map(|left| left.map(|left| op(left.as_i128(), right)))
+        .map(|left| left.map(|left| op(left, right)))
         .collect())
 }
 
@@ -152,7 +152,7 @@ where
         .zip(right.iter())
         .map(|(left, right)| {
             if let (Some(left), Some(right)) = (left, right) {
-                Some(op(left.as_i128(), right.as_i128()))
+                Some(op(left, right))
             } else {
                 None
             }
@@ -288,7 +288,7 @@ where
         .zip(right.iter())
         .map(|(left, right)| {
             if let (Some(left), Some(right)) = (left, right) {
-                Some(op(left.as_i128(), right.as_i128())).transpose()
+                Some(op(left, right)).transpose()
             } else {
                 Ok(None)
             }
@@ -307,7 +307,7 @@ where
     left.iter()
         .map(|left| {
             if let Some(left) = left {
-                Some(op(left.as_i128(), right)).transpose()
+                Some(op(left, right)).transpose()
             } else {
                 Ok(None)
             }
@@ -444,19 +444,15 @@ mod tests {
         precision: u8,
         scale: u8,
     ) -> Decimal128Array {
-        let mut decimal_builder =
-            Decimal128Builder::with_capacity(array.len(), precision, scale);
-        for value in array {
-            match value {
-                None => {
-                    decimal_builder.append_null();
-                }
-                Some(v) => {
-                    decimal_builder.append_value(*v).expect("valid value");
-                }
-            }
+        let mut decimal_builder = Decimal128Builder::with_capacity(array.len());
+
+        for value in array.iter().copied() {
+            decimal_builder.append_option(value)
         }
-        decimal_builder.finish()
+        decimal_builder
+            .finish()
+            .with_precision_and_scale(precision, scale)
+            .unwrap()
     }
 
     #[test]
