@@ -25,13 +25,14 @@ use arrow::{
     record_batch::RecordBatch,
 };
 
-use crate::PhysicalExpr;
-use datafusion_common::Result;
+use crate::physical_expr::down_cast_any_ref;
+use crate::{ExprBoundaries, PhysicalExpr, PhysicalExprStats};
 use datafusion_common::ScalarValue;
+use datafusion_common::{ColumnStatistics, Result};
 use datafusion_expr::{ColumnarValue, Expr};
 
 /// Represents a literal value
-#[derive(Debug)]
+#[derive(Debug, PartialEq, Eq)]
 pub struct Literal {
     value: ScalarValue,
 }
@@ -71,6 +72,49 @@ impl PhysicalExpr for Literal {
     fn evaluate(&self, _batch: &RecordBatch) -> Result<ColumnarValue> {
         Ok(ColumnarValue::Scalar(self.value.clone()))
     }
+
+    fn expr_stats(&self) -> Arc<dyn PhysicalExprStats> {
+        Arc::new(LiteralExprStats {
+            value: self.value.clone(),
+        })
+    }
+
+    fn children(&self) -> Vec<Arc<dyn PhysicalExpr>> {
+        vec![]
+    }
+
+    fn with_new_children(
+        self: Arc<Self>,
+        _children: Vec<Arc<dyn PhysicalExpr>>,
+    ) -> Result<Arc<dyn PhysicalExpr>> {
+        Ok(self)
+    }
+}
+
+impl PartialEq<dyn Any> for Literal {
+    fn eq(&self, other: &dyn Any) -> bool {
+        down_cast_any_ref(other)
+            .downcast_ref::<Self>()
+            .map(|x| self == x)
+            .unwrap_or(false)
+    }
+}
+
+struct LiteralExprStats {
+    value: ScalarValue,
+}
+
+impl PhysicalExprStats for LiteralExprStats {
+    #[allow(unused_variables)]
+    /// A literal's boundaries are the same as its value's boundaries (since it is a
+    /// scalar, both min/max are the same).
+    fn boundaries(&self, columns: &[ColumnStatistics]) -> Option<ExprBoundaries> {
+        Some(ExprBoundaries::new(
+            self.value.clone(),
+            self.value.clone(),
+            Some(1),
+        ))
+    }
 }
 
 /// Create a literal expression
@@ -110,4 +154,17 @@ mod tests {
 
         Ok(())
     }
+
+    #[test]
+    fn literal_stats() -> Result<()> {
+        let literal_expr = lit(42i32);
+        let stats = literal_expr.expr_stats();
+        let boundaries = stats.boundaries(&[]).unwrap();
+        assert_eq!(boundaries.min_value, ScalarValue::Int32(Some(42)));
+        assert_eq!(boundaries.max_value, ScalarValue::Int32(Some(42)));
+        assert_eq!(boundaries.distinct_count, Some(1));
+        assert_eq!(boundaries.selectivity, None);
+
+        Ok(())
+    }
 }
