@@ -23,13 +23,13 @@ mod case;
 mod cast;
 mod column;
 mod datetime;
-mod delta;
 mod get_indexed_field;
 mod in_list;
 mod is_not_null;
 mod is_null;
 mod literal;
 mod negative;
+mod no_op;
 mod not;
 mod nullif;
 mod try_cast;
@@ -80,6 +80,7 @@ pub use is_not_null::{is_not_null, IsNotNullExpr};
 pub use is_null::{is_null, IsNullExpr};
 pub use literal::{lit, Literal};
 pub use negative::{negative, NegativeExpr};
+pub use no_op::NoOp;
 pub use not::{not, NotExpr};
 pub use nullif::nullif_func;
 pub use try_cast::{try_cast, TryCastExpr};
