@@ -19,6 +19,7 @@ use std::any::Any;
 use std::fmt;
 use std::sync::Arc;
 
+use crate::physical_expr::down_cast_any_ref;
 use crate::PhysicalExpr;
 use arrow::compute;
 use arrow::compute::kernels;
@@ -90,6 +91,29 @@ impl PhysicalExpr for TryCastExpr {
             }
         }
     }
+
+    fn children(&self) -> Vec<Arc<dyn PhysicalExpr>> {
+        vec![self.expr.clone()]
+    }
+
+    fn with_new_children(
+        self: Arc<Self>,
+        children: Vec<Arc<dyn PhysicalExpr>>,
+    ) -> Result<Arc<dyn PhysicalExpr>> {
+        Ok(Arc::new(TryCastExpr::new(
+            children[0].clone(),
+            self.cast_type.clone(),
+        )))
+    }
+}
+
+impl PartialEq<dyn Any> for TryCastExpr {
+    fn eq(&self, other: &dyn Any) -> bool {
+        down_cast_any_ref(other)
+            .downcast_ref::<Self>()
+            .map(|x| self.expr.eq(&x.expr) && self.cast_type == x.cast_type)
+            .unwrap_or(false)
+    }
 }
 
 /// Return a PhysicalExpression representing `expr` casted to
@@ -107,7 +131,7 @@ pub fn try_cast(
     } else if can_cast_types(&expr_type, &cast_type) {
         Ok(Arc::new(TryCastExpr::new(expr, cast_type)))
     } else {
-        Err(DataFusionError::Internal(format!(
+        Err(DataFusionError::NotImplemented(format!(
             "Unsupported CAST from {:?} to {:?}",
             expr_type, cast_type
         )))
@@ -121,7 +145,6 @@ mod tests {
     use arrow::array::{
         Decimal128Array, Decimal128Builder, StringArray, Time64NanosecondArray,
     };
-    use arrow::util::decimal::Decimal128;
     use arrow::{
         array::{
             Array, Float32Array, Float64Array, Int16Array, Int32Array, Int64Array,
@@ -233,37 +256,28 @@ mod tests {
         // try cast one decimal data type to another decimal data type
         let array: Vec<i128> = vec![1234, 2222, 3, 4000, 5000];
         let decimal_array = create_decimal_array(&array, 10, 3);
-        let convert = |v: i128| Decimal128::new(20, 6, &v.to_le_bytes());
         generic_decimal_to_other_test_cast!(
             decimal_array,
             DataType::Decimal128(10, 3),
             Decimal128Array,
             DataType::Decimal128(20, 6),
             vec![
-                Some(convert(1_234_000)),
-                Some(convert(2_222_000)),
-                Some(convert(3_000)),
-                Some(convert(4_000_000)),
-                Some(convert(5_000_000)),
+                Some(1_234_000),
+                Some(2_222_000),
+                Some(3_000),
+                Some(4_000_000),
+                Some(5_000_000),
                 None,
             ]
         );
 
         let decimal_array = create_decimal_array(&array, 10, 3);
-        let convert = |v: i128| Decimal128::new(10, 2, &v.to_le_bytes());
         generic_decimal_to_other_test_cast!(
             decimal_array,
             DataType::Decimal128(10, 3),
             Decimal128Array,
             DataType::Decimal128(10, 2),
-            vec![
-                Some(convert(123)),
-                Some(convert(222)),
-                Some(convert(0)),
-                Some(convert(400)),
-                Some(convert(500)),
-                None,
-            ]
+            vec![Some(123), Some(222), Some(0), Some(400), Some(500), None,]
         );
 
         Ok(())
@@ -382,109 +396,66 @@ mod tests {
     #[test]
     fn test_try_cast_numeric_to_decimal() -> Result<()> {
         // int8
-        let convert = |v: i128| Decimal128::new(3, 0, &v.to_le_bytes());
         generic_test_cast!(
             Int8Array,
             DataType::Int8,
             vec![1, 2, 3, 4, 5],
             Decimal128Array,
             DataType::Decimal128(3, 0),
-            vec![
-                Some(convert(1)),
-                Some(convert(2)),
-                Some(convert(3)),
-                Some(convert(4)),
-                Some(convert(5)),
-            ]
+            vec![Some(1), Some(2), Some(3), Some(4), Some(5),]
         );
 
         // int16
-        let convert = |v: i128| Decimal128::new(5, 0, &v.to_le_bytes());
         generic_test_cast!(
             Int16Array,
             DataType::Int16,
             vec![1, 2, 3, 4, 5],
             Decimal128Array,
             DataType::Decimal128(5, 0),
-            vec![
-                Some(convert(1)),
-                Some(convert(2)),
-                Some(convert(3)),
-                Some(convert(4)),
-                Some(convert(5)),
-            ]
+            vec![Some(1), Some(2), Some(3), Some(4), Some(5),]
         );
 
         // int32
-        let convert = |v: i128| Decimal128::new(10, 0, &v.to_le_bytes());
         generic_test_cast!(
             Int32Array,
             DataType::Int32,
             vec![1, 2, 3, 4, 5],
             Decimal128Array,
             DataType::Decimal128(10, 0),
-            vec![
-                Some(convert(1)),
-                Some(convert(2)),
-                Some(convert(3)),
-                Some(convert(4)),
-                Some(convert(5)),
-            ]
+            vec![Some(1), Some(2), Some(3), Some(4), Some(5),]
         );
 
         // int64
-        let convert = |v: i128| Decimal128::new(20, 0, &v.to_le_bytes());
         generic_test_cast!(
             Int64Array,
             DataType::Int64,
             vec![1, 2, 3, 4, 5],
             Decimal128Array,
             DataType::Decimal128(20, 0),
-            vec![
-                Some(convert(1)),
-                Some(convert(2)),
-                Some(convert(3)),
-                Some(convert(4)),
-                Some(convert(5)),
-            ]
+            vec![Some(1), Some(2), Some(3), Some(4), Some(5),]
         );
 
         // int64 to different scale
-        let convert = |v: i128| Decimal128::new(20, 2, &v.to_le_bytes());
         generic_test_cast!(
             Int64Array,
             DataType::Int64,
             vec![1, 2, 3, 4, 5],
             Decimal128Array,
             DataType::Decimal128(20, 2),
-            vec![
-                Some(convert(100)),
-                Some(convert(200)),
-                Some(convert(300)),
-                Some(convert(400)),
-                Some(convert(500)),
-            ]
+            vec![Some(100), Some(200), Some(300), Some(400), Some(500),]
         );
 
         // float32
-        let convert = |v: i128| Decimal128::new(10, 2, &v.to_le_bytes());
         generic_test_cast!(
             Float32Array,
             DataType::Float32,
             vec![1.5, 2.5, 3.0, 1.123_456_8, 5.50],
             Decimal128Array,
             DataType::Decimal128(10, 2),
-            vec![
-                Some(convert(150)),
-                Some(convert(250)),
-                Some(convert(300)),
-                Some(convert(112)),
-                Some(convert(550)),
-            ]
+            vec![Some(150), Some(250), Some(300), Some(112), Some(550),]
         );
 
         // float64
-        let convert = |v: i128| Decimal128::new(20, 4, &v.to_le_bytes());
         generic_test_cast!(
             Float64Array,
             DataType::Float64,
@@ -492,11 +463,11 @@ mod tests {
             Decimal128Array,
             DataType::Decimal128(20, 4),
             vec![
-                Some(convert(15000)),
-                Some(convert(25000)),
-                Some(convert(30000)),
-                Some(convert(11234)),
-                Some(convert(55000)),
+                Some(15000),
+                Some(25000),
+                Some(30000),
+                Some(11234),
+                Some(55000),
             ]
         );
         Ok(())
@@ -577,12 +548,14 @@ mod tests {
 
     // create decimal array with the specified precision and scale
     fn create_decimal_array(array: &[i128], precision: u8, scale: u8) -> Decimal128Array {
-        let mut decimal_builder =
-            Decimal128Builder::with_capacity(array.len(), precision, scale);
+        let mut decimal_builder = Decimal128Builder::with_capacity(array.len());
         for value in array {
-            decimal_builder.append_value(*value).expect("valid value");
+            decimal_builder.append_value(*value);
         }
         decimal_builder.append_null();
-        decimal_builder.finish()
+        decimal_builder
+            .finish()
+            .with_precision_and_scale(precision, scale)
+            .unwrap()
     }
 }
