@@ -24,11 +24,13 @@ pub mod datetime_expressions;
 pub mod execution_props;
 pub mod expressions;
 pub mod functions;
+pub mod hash_utils;
 pub mod math_expressions;
 mod physical_expr;
 pub mod planner;
 #[cfg(feature = "regex_expressions")]
 pub mod regex_expressions;
+pub mod rewrite;
 mod scalar_function;
 mod sort_expr;
 pub mod string_expressions;
@@ -37,13 +39,14 @@ pub mod type_coercion;
 pub mod udf;
 #[cfg(feature = "unicode_expressions")]
 pub mod unicode_expressions;
+pub mod utils;
 pub mod var_provider;
 pub mod window;
 
 // reexport this to maintain compatibility with anything that used from_slice previously
 pub use aggregate::AggregateExpr;
 pub use datafusion_common::from_slice;
-pub use physical_expr::PhysicalExpr;
+pub use physical_expr::{ExprBoundaries, PhysicalExpr, PhysicalExprStats};
 pub use planner::create_physical_expr;
 pub use scalar_function::ScalarFunctionExpr;
 pub use sort_expr::PhysicalSortExpr;
