@@ -33,6 +33,12 @@ pub struct PhysicalSortExpr {
     pub options: SortOptions,
 }
 
+impl PartialEq for PhysicalSortExpr {
+    fn eq(&self, other: &PhysicalSortExpr) -> bool {
+        self.options == other.options && self.expr.eq(&other.expr)
+    }
+}
+
 impl std::fmt::Display for PhysicalSortExpr {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         let opts_string = match (self.options.descending, self.options.nulls_first) {
