@@ -32,7 +32,9 @@ use datafusion_common::ScalarValue;
 use datafusion_common::{DataFusionError, Result};
 use datafusion_expr::ColumnarValue;
 use std::any::type_name;
+use std::iter;
 use std::sync::Arc;
+use uuid::Uuid;
 
 macro_rules! downcast_string_arg {
     ($ARG:expr, $NAME:expr, $T:ident) => {{
@@ -586,3 +588,20 @@ where
 pub fn upper(args: &[ColumnarValue]) -> Result<ColumnarValue> {
     handle(args, |string| string.to_ascii_uppercase(), "upper")
 }
+
+/// Prints random (v4) uuid values per row
+/// uuid() = 'a0eebc99-9c0b-4ef8-bb6d-6bb9bd380a11'
+pub fn uuid(args: &[ColumnarValue]) -> Result<ColumnarValue> {
+    let len: usize = match &args[0] {
+        ColumnarValue::Array(array) => array.len(),
+        _ => {
+            return Err(DataFusionError::Internal(
+                "Expect uuid function to take no param".to_string(),
+            ))
+        }
+    };
+
+    let values = iter::repeat_with(|| Uuid::new_v4().to_string()).take(len);
+    let array = GenericStringArray::<i32>::from_iter_values(values);
+    Ok(ColumnarValue::Array(Arc::new(array)))
+}
