@@ -51,7 +51,7 @@ mod roundtrip_tests {
         logical_plan_to_bytes, logical_plan_to_bytes_with_extension_codec,
     };
     use crate::logical_plan::LogicalExtensionCodec;
-    use arrow::datatypes::Schema;
+    use arrow::datatypes::{Schema, SchemaRef};
     use arrow::{
         array::ArrayRef,
         datatypes::{
@@ -59,18 +59,26 @@ mod roundtrip_tests {
             TimeUnit, UnionMode,
         },
     };
-    use datafusion::logical_plan::create_udaf;
+    use datafusion::datasource::datasource::TableProviderFactory;
+    use datafusion::datasource::TableProvider;
+    use datafusion::execution::runtime_env::{RuntimeConfig, RuntimeEnv};
     use datafusion::physical_plan::functions::make_scalar_function;
-    use datafusion::prelude::{create_udf, CsvReadOptions, SessionContext};
+    use datafusion::prelude::{
+        create_udf, CsvReadOptions, SessionConfig, SessionContext,
+    };
+    use datafusion::test_util::{TestTableFactory, TestTableProvider};
     use datafusion_common::{DFSchemaRef, DataFusionError, ScalarValue};
-    use datafusion_expr::expr::GroupingSet;
+    use datafusion_expr::create_udaf;
+    use datafusion_expr::expr::{Between, BinaryExpr, Case, Cast, GroupingSet, Like};
     use datafusion_expr::logical_plan::{Extension, UserDefinedLogicalNode};
     use datafusion_expr::{
         col, lit, Accumulator, AggregateFunction, AggregateState,
-        BuiltinScalarFunction::Sqrt, Expr, LogicalPlan, Operator, Volatility,
+        BuiltinScalarFunction::{Sqrt, Substr},
+        Expr, LogicalPlan, Operator, Volatility,
     };
     use prost::Message;
     use std::any::Any;
+    use std::collections::HashMap;
     use std::fmt;
     use std::fmt::Debug;
     use std::fmt::Formatter;
@@ -129,6 +137,92 @@ mod roundtrip_tests {
         Ok(())
     }
 
+    #[derive(Clone, PartialEq, Eq, ::prost::Message)]
+    pub struct TestTableProto {
+        /// URL of the table root
+        #[prost(string, tag = "1")]
+        pub url: String,
+    }
+
+    #[derive(Debug)]
+    pub struct TestTableProviderCodec {}
+
+    impl LogicalExtensionCodec for TestTableProviderCodec {
+        fn try_decode(
+            &self,
+            _buf: &[u8],
+            _inputs: &[LogicalPlan],
+            _ctx: &SessionContext,
+        ) -> Result<Extension, DataFusionError> {
+            Err(DataFusionError::NotImplemented(
+                "No extension codec provided".to_string(),
+            ))
+        }
+
+        fn try_encode(
+            &self,
+            _node: &Extension,
+            _buf: &mut Vec<u8>,
+        ) -> Result<(), DataFusionError> {
+            Err(DataFusionError::NotImplemented(
+                "No extension codec provided".to_string(),
+            ))
+        }
+
+        fn try_decode_table_provider(
+            &self,
+            buf: &[u8],
+            _schema: SchemaRef,
+            _ctx: &SessionContext,
+        ) -> Result<Arc<dyn TableProvider>, DataFusionError> {
+            let msg = TestTableProto::decode(buf).map_err(|_| {
+                DataFusionError::Internal("Error decoding test table".to_string())
+            })?;
+            let provider = TestTableProvider { url: msg.url };
+            Ok(Arc::new(provider))
+        }
+
+        fn try_encode_table_provider(
+            &self,
+            node: Arc<dyn TableProvider>,
+            buf: &mut Vec<u8>,
+        ) -> Result<(), DataFusionError> {
+            let table = node
+                .as_ref()
+                .as_any()
+                .downcast_ref::<TestTableProvider>()
+                .expect("Can't encode non-test tables");
+            let msg = TestTableProto {
+                url: table.url.clone(),
+            };
+            msg.encode(buf).map_err(|_| {
+                DataFusionError::Internal("Error encoding test table".to_string())
+            })
+        }
+    }
+
+    #[tokio::test]
+    async fn roundtrip_custom_tables() -> Result<(), DataFusionError> {
+        let mut table_factories: HashMap<String, Arc<dyn TableProviderFactory>> =
+            HashMap::new();
+        table_factories.insert("testtable".to_string(), Arc::new(TestTableFactory {}));
+        let cfg = RuntimeConfig::new().with_table_factories(table_factories);
+        let env = RuntimeEnv::new(cfg).unwrap();
+        let ses = SessionConfig::new();
+        let ctx = SessionContext::with_config_rt(ses, Arc::new(env));
+
+        let sql = "CREATE EXTERNAL TABLE t STORED AS testtable LOCATION 's3://bucket/schema/table';";
+        ctx.sql(sql).await.unwrap();
+
+        let codec = TestTableProviderCodec {};
+        let scan = ctx.table("t")?.to_logical_plan()?;
+        let bytes = logical_plan_to_bytes_with_extension_codec(&scan, &codec)?;
+        let logical_round_trip =
+            logical_plan_from_bytes_with_extension_codec(&bytes, &ctx, &codec)?;
+        assert_eq!(format!("{:?}", scan), format!("{:?}", logical_round_trip));
+        Ok(())
+    }
+
     #[tokio::test]
     async fn roundtrip_logical_plan_aggregation() -> Result<(), DataFusionError> {
         let ctx = SessionContext::new();
@@ -158,6 +252,34 @@ mod roundtrip_tests {
         Ok(())
     }
 
+    #[tokio::test]
+    async fn roundtrip_single_count_distinct() -> Result<(), DataFusionError> {
+        let ctx = SessionContext::new();
+
+        let schema = Schema::new(vec![
+            Field::new("a", DataType::Int64, true),
+            Field::new("b", DataType::Decimal128(15, 2), true),
+        ]);
+
+        ctx.register_csv(
+            "t1",
+            "testdata/test.csv",
+            CsvReadOptions::default().schema(&schema),
+        )
+        .await?;
+
+        let query = "SELECT a, COUNT(DISTINCT b) as b_cd FROM t1 GROUP BY a";
+        let plan = ctx.sql(query).await?.to_logical_plan()?;
+
+        println!("{:?}", plan);
+
+        let bytes = logical_plan_to_bytes(&plan)?;
+        let logical_round_trip = logical_plan_from_bytes(&bytes, &ctx)?;
+        assert_eq!(format!("{:?}", plan), format!("{:?}", logical_round_trip));
+
+        Ok(())
+    }
+
     #[tokio::test]
     async fn roundtrip_logical_plan_with_extension() -> Result<(), DataFusionError> {
         let ctx = SessionContext::new();
@@ -170,6 +292,20 @@ mod roundtrip_tests {
         Ok(())
     }
 
+    #[tokio::test]
+    async fn roundtrip_logical_plan_with_view_scan() -> Result<(), DataFusionError> {
+        let ctx = SessionContext::new();
+        ctx.register_csv("t1", "testdata/test.csv", CsvReadOptions::default())
+            .await?;
+        ctx.sql("CREATE VIEW view_t1(a, b) AS SELECT a, b FROM t1")
+            .await?;
+        let plan = ctx.sql("SELECT * FROM view_t1").await?.to_logical_plan()?;
+        let bytes = logical_plan_to_bytes(&plan)?;
+        let logical_round_trip = logical_plan_from_bytes(&bytes, &ctx)?;
+        assert_eq!(format!("{:?}", plan), format!("{:?}", logical_round_trip));
+        Ok(())
+    }
+
     pub mod proto {
         #[derive(Clone, PartialEq, ::prost::Message)]
         pub struct TopKPlanProto {
@@ -310,6 +446,27 @@ mod roundtrip_tests {
                 ))
             }
         }
+
+        fn try_decode_table_provider(
+            &self,
+            _buf: &[u8],
+            _schema: SchemaRef,
+            _ctx: &SessionContext,
+        ) -> Result<Arc<dyn TableProvider>, DataFusionError> {
+            Err(DataFusionError::Internal(
+                "unsupported plan type".to_string(),
+            ))
+        }
+
+        fn try_encode_table_provider(
+            &self,
+            _node: Arc<dyn TableProvider>,
+            _buf: &mut Vec<u8>,
+        ) -> Result<(), DataFusionError> {
+            Err(DataFusionError::Internal(
+                "unsupported plan type".to_string(),
+            ))
+        }
     }
 
     #[test]
@@ -320,7 +477,7 @@ mod roundtrip_tests {
                 Some(vec![]),
                 Box::new(vec![Field::new("item", DataType::Int16, true)]),
             ),
-            // Should fail due to inconsistent types
+            // Should fail due to inconsistent types in the list
             ScalarValue::new_list(
                 Some(vec![
                     ScalarValue::Int16(None),
@@ -335,6 +492,13 @@ mod roundtrip_tests {
                 ]),
                 DataType::List(new_box_field("item", DataType::Int16, true)),
             ),
+            ScalarValue::new_list(
+                Some(vec![
+                    ScalarValue::Float32(None),
+                    ScalarValue::Float32(Some(32.0)),
+                ]),
+                DataType::Int16,
+            ),
             ScalarValue::new_list(
                 Some(vec![
                     ScalarValue::new_list(
@@ -369,15 +533,20 @@ mod roundtrip_tests {
         ];
 
         for test_case in should_fail_on_seralize.into_iter() {
-            let res: std::result::Result<
-                super::protobuf::ScalarValue,
-                super::to_proto::Error,
-            > = (&test_case).try_into();
-            assert!(
-                res.is_err(),
-                "The value {:?} should not have been able to serialize. Serialized to :{:?}",
-                test_case, res
-            );
+            let proto: Result<super::protobuf::ScalarValue, super::to_proto::Error> =
+                (&test_case).try_into();
+
+            // Validation is also done on read, so if serialization passed
+            // also try to convert back to ScalarValue
+            if let Ok(proto) = proto {
+                let res: Result<ScalarValue, _> = (&proto).try_into();
+                assert!(
+                    res.is_err(),
+                    "The value {:?} unexpectedly serialized without error:{:?}",
+                    test_case,
+                    res
+                );
+            }
         }
     }
 
@@ -482,14 +651,11 @@ mod roundtrip_tests {
                     ScalarValue::Float32(Some(2.0)),
                     ScalarValue::Float32(Some(1.0)),
                 ]),
-                DataType::List(new_box_field("level1", DataType::Float32, true)),
+                DataType::Float32,
             ),
             ScalarValue::new_list(
                 Some(vec![
-                    ScalarValue::new_list(
-                        None,
-                        DataType::List(new_box_field("level2", DataType::Float32, true)),
-                    ),
+                    ScalarValue::new_list(None, DataType::Float32),
                     ScalarValue::new_list(
                         Some(vec![
                             ScalarValue::Float32(Some(-213.1)),
@@ -498,14 +664,10 @@ mod roundtrip_tests {
                             ScalarValue::Float32(Some(2.0)),
                             ScalarValue::Float32(Some(1.0)),
                         ]),
-                        DataType::List(new_box_field("level2", DataType::Float32, true)),
+                        DataType::Float32,
                     ),
                 ]),
-                DataType::List(new_box_field(
-                    "level1",
-                    DataType::List(new_box_field("level2", DataType::Float32, true)),
-                    true,
-                )),
+                DataType::List(new_box_field("item", DataType::Float32, true)),
             ),
             ScalarValue::Dictionary(
                 Box::new(DataType::Int32),
@@ -536,6 +698,12 @@ mod roundtrip_tests {
                     Field::new("a", DataType::Boolean, false),
                 ]),
             ),
+            ScalarValue::FixedSizeBinary(
+                b"bar".to_vec().len() as i32,
+                Some(b"bar".to_vec()),
+            ),
+            ScalarValue::FixedSizeBinary(0, None),
+            ScalarValue::FixedSizeBinary(5, None),
         ];
 
         for test_case in should_pass.into_iter() {
@@ -576,149 +744,20 @@ mod roundtrip_tests {
             DataType::Utf8,
             DataType::LargeUtf8,
             // Recursive list tests
-            DataType::List(new_box_field("Level1", DataType::Boolean, true)),
+            DataType::List(new_box_field("level1", DataType::Boolean, true)),
             DataType::List(new_box_field(
                 "Level1",
-                DataType::List(new_box_field("Level2", DataType::Date32, true)),
+                DataType::List(new_box_field("level2", DataType::Date32, true)),
                 true,
             )),
         ];
 
-        let should_fail: Vec<DataType> = vec![
-            DataType::Null,
-            DataType::Float16,
-            // Add more timestamp tests
-            DataType::Timestamp(TimeUnit::Millisecond, None),
-            DataType::Date64,
-            DataType::Time32(TimeUnit::Second),
-            DataType::Time32(TimeUnit::Millisecond),
-            DataType::Time32(TimeUnit::Microsecond),
-            DataType::Time32(TimeUnit::Nanosecond),
-            DataType::Time64(TimeUnit::Second),
-            DataType::Time64(TimeUnit::Millisecond),
-            DataType::Duration(TimeUnit::Second),
-            DataType::Duration(TimeUnit::Millisecond),
-            DataType::Duration(TimeUnit::Microsecond),
-            DataType::Duration(TimeUnit::Nanosecond),
-            DataType::Interval(IntervalUnit::YearMonth),
-            DataType::Interval(IntervalUnit::DayTime),
-            DataType::Binary,
-            DataType::FixedSizeBinary(0),
-            DataType::FixedSizeBinary(1234),
-            DataType::FixedSizeBinary(-432),
-            DataType::LargeBinary,
-            DataType::Decimal128(123, 234),
-            // Recursive list tests
-            DataType::List(new_box_field("Level1", DataType::Binary, true)),
-            DataType::List(new_box_field(
-                "Level1",
-                DataType::List(new_box_field(
-                    "Level2",
-                    DataType::FixedSizeBinary(53),
-                    false,
-                )),
-                true,
-            )),
-            // Fixed size lists
-            DataType::FixedSizeList(new_box_field("Level1", DataType::Binary, true), 4),
-            DataType::FixedSizeList(
-                new_box_field(
-                    "Level1",
-                    DataType::List(new_box_field(
-                        "Level2",
-                        DataType::FixedSizeBinary(53),
-                        false,
-                    )),
-                    true,
-                ),
-                41,
-            ),
-            // Struct Testing
-            DataType::Struct(vec![
-                Field::new("nullable", DataType::Boolean, false),
-                Field::new("name", DataType::Utf8, false),
-                Field::new("datatype", DataType::Binary, false),
-            ]),
-            DataType::Struct(vec![
-                Field::new("nullable", DataType::Boolean, false),
-                Field::new("name", DataType::Utf8, false),
-                Field::new("datatype", DataType::Binary, false),
-                Field::new(
-                    "nested_struct",
-                    DataType::Struct(vec![
-                        Field::new("nullable", DataType::Boolean, false),
-                        Field::new("name", DataType::Utf8, false),
-                        Field::new("datatype", DataType::Binary, false),
-                    ]),
-                    true,
-                ),
-            ]),
-            DataType::Union(
-                vec![
-                    Field::new("nullable", DataType::Boolean, false),
-                    Field::new("name", DataType::Utf8, false),
-                    Field::new("datatype", DataType::Binary, false),
-                ],
-                vec![0, 2, 3],
-                UnionMode::Dense,
-            ),
-            DataType::Union(
-                vec![
-                    Field::new("nullable", DataType::Boolean, false),
-                    Field::new("name", DataType::Utf8, false),
-                    Field::new("datatype", DataType::Binary, false),
-                    Field::new(
-                        "nested_struct",
-                        DataType::Struct(vec![
-                            Field::new("nullable", DataType::Boolean, false),
-                            Field::new("name", DataType::Utf8, false),
-                            Field::new("datatype", DataType::Binary, false),
-                        ]),
-                        true,
-                    ),
-                ],
-                vec![1, 2, 3],
-                UnionMode::Sparse,
-            ),
-            DataType::Dictionary(
-                Box::new(DataType::Utf8),
-                Box::new(DataType::Struct(vec![
-                    Field::new("nullable", DataType::Boolean, false),
-                    Field::new("name", DataType::Utf8, false),
-                    Field::new("datatype", DataType::Binary, false),
-                ])),
-            ),
-            DataType::Dictionary(
-                Box::new(DataType::Decimal128(10, 50)),
-                Box::new(DataType::FixedSizeList(
-                    new_box_field("Level1", DataType::Binary, true),
-                    4,
-                )),
-            ),
-        ];
-
         for test_case in should_pass.into_iter() {
             let field = Field::new("item", test_case, true);
             let proto: super::protobuf::Field = (&field).try_into().unwrap();
             let roundtrip: Field = (&proto).try_into().unwrap();
             assert_eq!(format!("{:?}", field), format!("{:?}", roundtrip));
         }
-
-        let mut success: Vec<DataType> = Vec::new();
-        for test_case in should_fail.into_iter() {
-            let proto: std::result::Result<
-                super::protobuf::ScalarType,
-                super::to_proto::Error,
-            > = (&Field::new("item", test_case.clone(), true)).try_into();
-            if proto.is_ok() {
-                success.push(test_case)
-            }
-        }
-        assert!(
-            success.is_empty(),
-            "These should have resulted in an error but completed successfully: {:?}",
-            success
-        );
     }
 
     #[test]
@@ -922,12 +961,12 @@ mod roundtrip_tests {
 
     #[test]
     fn roundtrip_between() {
-        let test_expr = Expr::Between {
-            expr: Box::new(lit(1.0_f32)),
-            negated: true,
-            low: Box::new(lit(2.0_f32)),
-            high: Box::new(lit(3.0_f32)),
-        };
+        let test_expr = Expr::Between(Between::new(
+            Box::new(lit(1.0_f32)),
+            true,
+            Box::new(lit(2.0_f32)),
+            Box::new(lit(3.0_f32)),
+        ));
 
         let ctx = SessionContext::new();
         roundtrip_expr_test(test_expr, ctx);
@@ -936,11 +975,11 @@ mod roundtrip_tests {
     #[test]
     fn roundtrip_binary_op() {
         fn test(op: Operator) {
-            let test_expr = Expr::BinaryExpr {
-                left: Box::new(lit(1.0_f32)),
+            let test_expr = Expr::BinaryExpr(BinaryExpr::new(
+                Box::new(lit(1.0_f32)),
                 op,
-                right: Box::new(lit(2.0_f32)),
-            };
+                Box::new(lit(2.0_f32)),
+            ));
             let ctx = SessionContext::new();
             roundtrip_expr_test(test_expr, ctx);
         }
@@ -970,11 +1009,11 @@ mod roundtrip_tests {
 
     #[test]
     fn roundtrip_case() {
-        let test_expr = Expr::Case {
-            expr: Some(Box::new(lit(1.0_f32))),
-            when_then_expr: vec![(Box::new(lit(2.0_f32)), Box::new(lit(3.0_f32)))],
-            else_expr: Some(Box::new(lit(4.0_f32))),
-        };
+        let test_expr = Expr::Case(Case::new(
+            Some(Box::new(lit(1.0_f32))),
+            vec![(Box::new(lit(2.0_f32)), Box::new(lit(3.0_f32)))],
+            Some(Box::new(lit(4.0_f32))),
+        ));
 
         let ctx = SessionContext::new();
         roundtrip_expr_test(test_expr, ctx);
@@ -982,11 +1021,11 @@ mod roundtrip_tests {
 
     #[test]
     fn roundtrip_case_with_null() {
-        let test_expr = Expr::Case {
-            expr: Some(Box::new(lit(1.0_f32))),
-            when_then_expr: vec![(Box::new(lit(2.0_f32)), Box::new(lit(3.0_f32)))],
-            else_expr: Some(Box::new(Expr::Literal(ScalarValue::Null))),
-        };
+        let test_expr = Expr::Case(Case::new(
+            Some(Box::new(lit(1.0_f32))),
+            vec![(Box::new(lit(2.0_f32)), Box::new(lit(3.0_f32)))],
+            Some(Box::new(Expr::Literal(ScalarValue::Null))),
+        ));
 
         let ctx = SessionContext::new();
         roundtrip_expr_test(test_expr, ctx);
@@ -1002,10 +1041,7 @@ mod roundtrip_tests {
 
     #[test]
     fn roundtrip_cast() {
-        let test_expr = Expr::Cast {
-            expr: Box::new(lit(1.0_f32)),
-            data_type: DataType::Boolean,
-        };
+        let test_expr = Expr::Cast(Cast::new(Box::new(lit(1.0_f32)), DataType::Boolean));
 
         let ctx = SessionContext::new();
         roundtrip_expr_test(test_expr, ctx);
@@ -1064,12 +1100,12 @@ mod roundtrip_tests {
     #[test]
     fn roundtrip_like() {
         fn like(negated: bool, escape_char: Option<char>) {
-            let test_expr = Expr::Like {
+            let test_expr = Expr::Like(Like::new(
                 negated,
-                expr: Box::new(col("col")),
-                pattern: Box::new(lit("[0-9]+")),
+                Box::new(col("col")),
+                Box::new(lit("[0-9]+")),
                 escape_char,
-            };
+            ));
             let ctx = SessionContext::new();
             roundtrip_expr_test(test_expr, ctx);
         }
@@ -1082,12 +1118,12 @@ mod roundtrip_tests {
     #[test]
     fn roundtrip_ilike() {
         fn ilike(negated: bool, escape_char: Option<char>) {
-            let test_expr = Expr::ILike {
+            let test_expr = Expr::ILike(Like::new(
                 negated,
-                expr: Box::new(col("col")),
-                pattern: Box::new(lit("[0-9]+")),
+                Box::new(col("col")),
+                Box::new(lit("[0-9]+")),
                 escape_char,
-            };
+            ));
             let ctx = SessionContext::new();
             roundtrip_expr_test(test_expr, ctx);
         }
@@ -1100,12 +1136,12 @@ mod roundtrip_tests {
     #[test]
     fn roundtrip_similar_to() {
         fn similar_to(negated: bool, escape_char: Option<char>) {
-            let test_expr = Expr::SimilarTo {
+            let test_expr = Expr::SimilarTo(Like::new(
                 negated,
-                expr: Box::new(col("col")),
-                pattern: Box::new(lit("[0-9]+")),
+                Box::new(col("col")),
+                Box::new(lit("[0-9]+")),
                 escape_char,
-            };
+            ));
             let ctx = SessionContext::new();
             roundtrip_expr_test(test_expr, ctx);
         }
@@ -1259,4 +1295,23 @@ mod roundtrip_tests {
         let ctx = SessionContext::new();
         roundtrip_expr_test(test_expr, ctx);
     }
+
+    #[test]
+    fn roundtrip_substr() {
+        // substr(string, position)
+        let test_expr = Expr::ScalarFunction {
+            fun: Substr,
+            args: vec![col("col"), lit(1_i64)],
+        };
+
+        // substr(string, position, count)
+        let test_expr_with_count = Expr::ScalarFunction {
+            fun: Substr,
+            args: vec![col("col"), lit(1_i64), lit(1_i64)],
+        };
+
+        let ctx = SessionContext::new();
+        roundtrip_expr_test(test_expr, ctx.clone());
+        roundtrip_expr_test(test_expr_with_count, ctx);
+    }
 }
