@@ -21,7 +21,10 @@ use arrow::datatypes::{DataType, DECIMAL128_MAX_PRECISION, DECIMAL_DEFAULT_SCALE
 use sqlparser::ast::Ident;
 
 use datafusion_common::{DataFusionError, Result, ScalarValue};
-use datafusion_expr::expr::GroupingSet;
+use datafusion_expr::expr::Cast;
+use datafusion_expr::expr::{
+    Between, BinaryExpr, Case, GetIndexedField, GroupingSet, Like,
+};
 use datafusion_expr::utils::{expr_as_column_expr, find_column_exprs};
 use datafusion_expr::{Expr, LogicalPlan};
 use std::collections::HashMap;
@@ -193,7 +196,7 @@ where
                     .iter()
                     .map(|e| clone_with_replacement(e, replacement_fn))
                     .collect::<Result<Vec<_>>>()?,
-                window_frame: *window_frame,
+                window_frame: window_frame.clone(),
             }),
             Expr::AggregateUDF { fun, args, filter } => Ok(Expr::AggregateUDF {
                 fun: fun.clone(),
@@ -207,17 +210,17 @@ where
                 Box::new(clone_with_replacement(nested_expr, replacement_fn)?),
                 alias_name.clone(),
             )),
-            Expr::Between {
-                expr: nested_expr,
+            Expr::Between(Between {
+                expr,
                 negated,
                 low,
                 high,
-            } => Ok(Expr::Between {
-                expr: Box::new(clone_with_replacement(nested_expr, replacement_fn)?),
-                negated: *negated,
-                low: Box::new(clone_with_replacement(low, replacement_fn)?),
-                high: Box::new(clone_with_replacement(high, replacement_fn)?),
-            }),
+            }) => Ok(Expr::Between(Between::new(
+                Box::new(clone_with_replacement(expr, replacement_fn)?),
+                *negated,
+                Box::new(clone_with_replacement(low, replacement_fn)?),
+                Box::new(clone_with_replacement(high, replacement_fn)?),
+            ))),
             Expr::InList {
                 expr: nested_expr,
                 list,
@@ -230,56 +233,54 @@ where
                     .collect::<Result<Vec<Expr>>>()?,
                 negated: *negated,
             }),
-            Expr::BinaryExpr { left, right, op } => Ok(Expr::BinaryExpr {
-                left: Box::new(clone_with_replacement(left, replacement_fn)?),
-                op: *op,
-                right: Box::new(clone_with_replacement(right, replacement_fn)?),
-            }),
-            Expr::Like {
+            Expr::BinaryExpr(BinaryExpr { left, right, op }) => {
+                Ok(Expr::BinaryExpr(BinaryExpr::new(
+                    Box::new(clone_with_replacement(left, replacement_fn)?),
+                    *op,
+                    Box::new(clone_with_replacement(right, replacement_fn)?),
+                )))
+            }
+            Expr::Like(Like {
                 negated,
                 expr,
                 pattern,
                 escape_char,
-            } => Ok(Expr::Like {
-                negated: *negated,
-                expr: Box::new(clone_with_replacement(expr, replacement_fn)?),
-                pattern: Box::new(clone_with_replacement(pattern, replacement_fn)?),
-                escape_char: *escape_char,
-            }),
-            Expr::ILike {
+            }) => Ok(Expr::Like(Like::new(
+                *negated,
+                Box::new(clone_with_replacement(expr, replacement_fn)?),
+                Box::new(clone_with_replacement(pattern, replacement_fn)?),
+                *escape_char,
+            ))),
+            Expr::ILike(Like {
                 negated,
                 expr,
                 pattern,
                 escape_char,
-            } => Ok(Expr::ILike {
-                negated: *negated,
-                expr: Box::new(clone_with_replacement(expr, replacement_fn)?),
-                pattern: Box::new(clone_with_replacement(pattern, replacement_fn)?),
-                escape_char: *escape_char,
-            }),
-            Expr::SimilarTo {
+            }) => Ok(Expr::ILike(Like::new(
+                *negated,
+                Box::new(clone_with_replacement(expr, replacement_fn)?),
+                Box::new(clone_with_replacement(pattern, replacement_fn)?),
+                *escape_char,
+            ))),
+            Expr::SimilarTo(Like {
                 negated,
                 expr,
                 pattern,
                 escape_char,
-            } => Ok(Expr::SimilarTo {
-                negated: *negated,
-                expr: Box::new(clone_with_replacement(expr, replacement_fn)?),
-                pattern: Box::new(clone_with_replacement(pattern, replacement_fn)?),
-                escape_char: *escape_char,
-            }),
-            Expr::Case {
-                expr: case_expr_opt,
-                when_then_expr,
-                else_expr: else_expr_opt,
-            } => Ok(Expr::Case {
-                expr: match case_expr_opt {
+            }) => Ok(Expr::SimilarTo(Like::new(
+                *negated,
+                Box::new(clone_with_replacement(expr, replacement_fn)?),
+                Box::new(clone_with_replacement(pattern, replacement_fn)?),
+                *escape_char,
+            ))),
+            Expr::Case(case) => Ok(Expr::Case(Case::new(
+                match &case.expr {
                     Some(case_expr) => {
                         Some(Box::new(clone_with_replacement(case_expr, replacement_fn)?))
                     }
                     None => None,
                 },
-                when_then_expr: when_then_expr
+                case.when_then_expr
                     .iter()
                     .map(|(a, b)| {
                         Ok((
@@ -288,13 +289,13 @@ where
                         ))
                     })
                     .collect::<Result<Vec<(_, _)>>>()?,
-                else_expr: match else_expr_opt {
+                match &case.else_expr {
                     Some(else_expr) => {
                         Some(Box::new(clone_with_replacement(else_expr, replacement_fn)?))
                     }
                     None => None,
                 },
-            }),
+            ))),
             Expr::ScalarFunction { fun, args } => Ok(Expr::ScalarFunction {
                 fun: fun.clone(),
                 args: args
@@ -340,13 +341,10 @@ where
             Expr::IsNotUnknown(nested_expr) => Ok(Expr::IsNotUnknown(Box::new(
                 clone_with_replacement(nested_expr, replacement_fn)?,
             ))),
-            Expr::Cast {
-                expr: nested_expr,
-                data_type,
-            } => Ok(Expr::Cast {
-                expr: Box::new(clone_with_replacement(nested_expr, replacement_fn)?),
-                data_type: data_type.clone(),
-            }),
+            Expr::Cast(Cast { expr, data_type }) => Ok(Expr::Cast(Cast::new(
+                Box::new(clone_with_replacement(expr, replacement_fn)?),
+                data_type.clone(),
+            ))),
             Expr::TryCast {
                 expr: nested_expr,
                 data_type,
@@ -379,10 +377,12 @@ where
             }),
             Expr::Wildcard => Ok(Expr::Wildcard),
             Expr::QualifiedWildcard { .. } => Ok(expr.clone()),
-            Expr::GetIndexedField { expr, key } => Ok(Expr::GetIndexedField {
-                expr: Box::new(clone_with_replacement(expr.as_ref(), replacement_fn)?),
-                key: key.clone(),
-            }),
+            Expr::GetIndexedField(GetIndexedField { key, expr }) => {
+                Ok(Expr::GetIndexedField(GetIndexedField::new(
+                    Box::new(clone_with_replacement(expr.as_ref(), replacement_fn)?),
+                    key.clone(),
+                )))
+            }
             Expr::GroupingSet(set) => match set {
                 GroupingSet::Rollup(exprs) => Ok(Expr::GroupingSet(GroupingSet::Rollup(
                     exprs
@@ -478,6 +478,16 @@ pub fn window_expr_common_partition_keys(window_exprs: &[Expr]) -> Result<&[Expr
         .iter()
         .map(|expr| match expr {
             Expr::WindowFunction { partition_by, .. } => Ok(partition_by),
+            Expr::Alias(expr, _) => {
+                // convert &Box<T> to &T
+                match &**expr {
+                    Expr::WindowFunction { partition_by, .. } => Ok(partition_by),
+                    expr => Err(DataFusionError::Execution(format!(
+                        "Impossibly got non-window expr {:?}",
+                        expr
+                    ))),
+                }
+            }
             expr => Err(DataFusionError::Execution(format!(
                 "Impossibly got non-window expr {:?}",
                 expr
@@ -512,9 +522,9 @@ pub(crate) fn make_decimal_type(
     };
 
     // Arrow decimal is i128 meaning 38 maximum decimal digits
-    if precision > DECIMAL128_MAX_PRECISION || scale > precision {
+    if precision == 0 || precision > DECIMAL128_MAX_PRECISION || scale > precision {
         Err(DataFusionError::Internal(format!(
-            "For decimal(precision, scale) precision must be less than or equal to 38 and scale can't be greater than precision. Got ({}, {})",
+            "Decimal(precision = {}, scale = {}) should satisty `0 < precision <= 38`, and `scale <= precision`.",
             precision, scale
         )))
     } else {
